import re
from urllib.parse import quote_plus, urljoin

from sickchill import logger
from sickchill.helper.common import convert_size, try_int
from sickchill.oldbeard import db, tvcache
from sickchill.oldbeard.bs4_parser import BS4Parser
from sickchill.oldbeard.common import Quality
from sickchill.oldbeard.name_parser.parser import InvalidNameException, InvalidShowException, NameParser
from sickchill.providers.torrent.TorrentProvider import TorrentProvider


class Provider(TorrentProvider):
    def __init__(self):
        super().__init__("ilCorsaroNero")

        categories = [  # Categories included in searches
            15,  # Serie TV
            5,  # Anime
            1,  # BDRip
            20,  # DVD
            19,  # Screener
        ]
        categories = ",".join(map(str, categories))

        self.url = "https://ilcorsaronero.link"
        self.urls = {
            "search": urljoin(self.url, "argh.php?search={0}&order=data&by=DESC&page={1}&category=" + categories),
        }

        self.public = True
        self.minseed = 0
        self.minleech = 0

        self.engrelease = None
        self.subtitle = None
        self.max_pages = 10

        self.proper_strings = ["PROPER", "REPACK"]
        self.sub_string = ["sub", "softsub"]

        self.hdtext = [
            " - Versione 720p",
            " Versione 720p",
            " V 720p",
            " V 720",
            " V HEVC",
            " V  HEVC",
            " V 1080",
            " Versione 1080p",
            " 720p HEVC",
            " Ver 720",
            " 720p HEVC",
            " 720p",
        ]

        self.cache = tvcache.TVCache(self, min_time=30)  # only poll ilCorsaroNero every 30 minutes max

    @staticmethod
    def _reverseQuality(quality):

        quality_string = ""

        if quality == Quality.SDTV:
            quality_string = " HDTV x264"
        if quality == Quality.SDDVD:
            quality_string = " DVDRIP"
        elif quality == Quality.HDTV:
            quality_string = " 720p HDTV x264"
        elif quality == Quality.FULLHDTV:
            quality_string = " 1080p HDTV x264"
        elif quality == Quality.RAWHDTV:
            quality_string = " 1080i HDTV mpeg2"
        elif quality == Quality.HDWEBDL:
            quality_string = " 720p WEB-DL h264"
        elif quality == Quality.FULLHDWEBDL:
            quality_string = " 1080p WEB-DL h264"
        elif quality == Quality.HDBLURAY:
            quality_string = " 720p Bluray x264"
        elif quality == Quality.FULLHDBLURAY:
            quality_string = " 1080p Bluray x264"

        return quality_string

    @staticmethod
    def _episodeQuality(torrent_rows):
        """
        Return The quality from the scene episode HTML row.
        """

        file_quality = (torrent_rows("td"))[1].find("a")["href"].replace("_", " ")
        logger.debug("Episode quality: {0}".format(file_quality))

        def checkName(options, func):
            return func([re.search(option, file_quality, re.I) for option in options])

        dvdOptions = checkName(["dvd", "dvdrip", "dvdmux", "DVD9", "DVD5"], any)
        bluRayOptions = checkName(["BD", "BDmux", "BDrip", "BRrip", "Bluray"], any)
        sdOptions = checkName(["h264", "divx", "XviD", "tv", "TVrip", "SATRip", "DTTrip", "Mpeg2"], any)
        hdOptions = checkName(["720p"], any)
        fullHD = checkName(["1080p", "fullHD"], any)
        webdl = checkName(["webdl", "webmux", "webrip", "dl-webmux", "web-dlmux", "webdl-mux", "web-dl", "webdlmux", "dlmux"], any)

        if sdOptions and not dvdOptions and not fullHD and not hdOptions:
            return Quality.SDTV
        elif dvdOptions:
            return Quality.SDDVD
        elif hdOptions and not bluRayOptions and not fullHD and not webdl:
            return Quality.HDTV
        elif not hdOptions and not bluRayOptions and fullHD and not webdl:
            return Quality.FULLHDTV
        elif hdOptions and not bluRayOptions and not fullHD and webdl:
            return Quality.HDWEBDL
        elif not hdOptions and not bluRayOptions and fullHD and webdl:
            return Quality.FULLHDWEBDL
        elif bluRayOptions and hdOptions and not fullHD:
            return Quality.HDBLURAY
        elif bluRayOptions and fullHD and not hdOptions:
            return Quality.FULLHDBLURAY
        else:
            return Quality.UNKNOWN

    def _is_italian(self, name):

        if not name or name == "None":
            return False

        subFound = italian = False
        for sub in self.sub_string:
            if re.search(sub, name, re.I):
                subFound = True
            else:
                continue

            if re.search("ita", name.split(sub)[0], re.I):
                logger.debug("Found Italian release: " + name)
                italian = True
                break

        if not subFound and re.search("ita", name, re.I):
            logger.debug("Found Italian release: " + name)
            italian = True

        return italian

    @staticmethod
    def _is_english(name):

        if not name or name == "None":
            return False

        english = False
        if re.search("eng", name, re.I):
            logger.debug("Found English release: " + name)
            english = True

        return english

    @staticmethod
    def _is_season_pack(name):

        try:
            parse_result = NameParser(tryIndexers=True).parse(name)
        except (InvalidNameException, InvalidShowException) as error:
            logger.debug("{0}".format(error))
            return False

        main_db_con = db.DBConnection()
        sql_selection = "select count(*) as count from tv_episodes where showid = ? and season = ?"
        episodes = main_db_con.select(sql_selection, [parse_result.show.indexerid, parse_result.season_number])
        if int(episodes[0]["count"]) == len(parse_result.episode_numbers):
            return True

    @staticmethod
    def _magnet_from_result(info_hash, title):
        return "magnet:?xt=urn:btih:{hash}&dn={title}&tr={trackers}".format(
            hash=info_hash, title=quote_plus(title), trackers="http://tracker.tntvillage.scambioetico.org:2710/announce"
        )

    def search(self, search_params, age=0, ep_obj=None):
        results = []

        for mode in search_params:
            items = []
            logger.debug(_("Search Mode: {mode}".format(mode=mode)))
            for search_string in search_params[mode]:
                if search_string == "":
                    continue

                search_string = str(search_string).replace(".", " ")
                logger.debug(_("Search String: {search_string}".format(search_string=search_string)))

                last_page = False
                for page in range(0, self.max_pages):
                    if last_page:
                        break

                    logger.debug("Processing page {0} of results".format(page))
                    search_url = self.urls["search"].format(search_string, page)

                    data = self.get_url(search_url, returns="text")
                    if not data:
                        logger.debug(_("No data returned from provider"))
                        continue

                    try:
                        with BS4Parser(data, "html5lib") as html:
                            table_header = html.find("tr", class_="bordo")
                            torrent_table = table_header.find_parent("table") if table_header else None
                            if not torrent_table:
                                logger.exception("Could not find table of torrents")
                                continue

                            torrent_rows = torrent_table("tr")

                            # Continue only if one Release is found
                            if len(torrent_rows) < 6 or len(torrent_rows[2]("td")) == 1:
                                logger.debug("Data returned from provider does not contain any torrents")
                                last_page = True
                                continue

                            if len(torrent_rows) < 45:
                                last_page = True

                            for result in torrent_rows[2:-3]:
                                result_cols = result("td")
                                if len(result_cols) == 1:
                                    # Ignore empty rows in the middle of the table
                                    continue
                                try:
                                    title = result("td")[1].get_text(strip=True)
                                    torrent_size = result("td")[2].get_text(strip=True)
                                    info_hash = result("td")[3].find("input", class_="downarrow")["value"].upper()
                                    download_url = self._magnet_from_result(info_hash, title)
                                    seeders = try_int(result("td")[5].get_text(strip=True))
                                    leechers = try_int(result("td")[6].get_text(strip=True))
                                    size = convert_size(torrent_size) or -1

                                except (AttributeError, IndexError, TypeError):
                                    continue

                                filename_qt = self._reverseQuality(self._episodeQuality(result))
                                for text in self.hdtext:
                                    title1 = title
                                    title = title.replace(text, filename_qt)
                                    if title != title1:
                                        break

                                if Quality.nameQuality(title) == Quality.UNKNOWN:
                                    title += filename_qt

                                if not self._is_italian(title) and not self.subtitle:
                                    logger.debug("Torrent is subtitled, skipping: {0}".format(title))
                                    continue

                                if self.engrelease and not self._is_english(title):
                                    logger.debug("Torrent isn't english audio/subtitled, skipping: {0}".format(title))
                                    continue

                                search_show = re.split(r"([Ss][\d{1,2}]+)", search_string)[0]
                                show_title = search_show
                                ep_params = ""
                                rindex = re.search(r"([Ss][\d{1,2}]+)", title)
                                if rindex:
                                    show_title = title[: rindex.start()]
                                    ep_params = title[rindex.start() :]
                                if show_title.lower() != search_show.lower() and search_show.lower() in show_title.lower():
                                    new_title = search_show + ep_params
                                    title = new_title

                                if not all([title, download_url]):
                                    continue

                                if self._is_season_pack(title):
                                    title = re.sub(r"([Ee][\d{1,2}\-?]+)", "", title)

                                # Filter unseeded torrent
                                if seeders < self.minseed or leechers < self.minleech:
                                    logger.debug(
                                        "Discarding torrent because it doesn't meet the minimum seeders or leechers: {0} (S:{1} L:{2})".format(
                                            title, seeders, leechers
                                        )
                                    )
                                    continue

                                item = {"title": title, "link": download_url, "size": size, "seeders": seeders, "leechers": leechers, "hash": info_hash}
                                if mode != "RSS":
                                    logger.debug(
                                        _(
                                            "Found result: {title} with {seeders} seeders and {leechers} leechers".format(
                                                title=title, seeders=seeders, leechers=leechers
                                            )
                                        )
                                    )

                                items.append(item)

                    except Exception as error:
                        logger.exception("Failed parsing provider. Error: {0}".format(error))

                # For each search mode sort all the items by seeders if available
                items.sort(key=lambda d: try_int(d.get("seeders", 0)), reverse=True)

                results += items

        return results



#!/usr/bin/python
# -- Content-Encoding: UTF-8 --
"""
Utility methods, for compatibility between Python version

:author: Thomas Calmant
:copyright: Copyright 2020, Thomas Calmant
:license: Apache License 2.0
:version: 0.4.1

..

    Copyright 2020 Thomas Calmant

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import sys

# ------------------------------------------------------------------------------

# Module version
__version_info__ = (0, 4, 1)
__version__ = ".".join(str(x) for x in __version_info__)

# Documentation strings format
__docformat__ = "restructuredtext en"

# ------------------------------------------------------------------------------

if sys.version_info[0] < 3:
    # Python 2
    # pylint: disable=E1101
    import types
    try:
        STRING_TYPES = (
            types.StringType,
            types.UnicodeType
        )
    except NameError:
        # Python built without unicode support
        STRING_TYPES = (types.StringType,)

    NUMERIC_TYPES = (
        types.IntType,
        types.LongType,
        types.FloatType
    )

    def to_bytes(string):
        """
        Converts the given string into bytes
        """
        # pylint: disable=E0602
        if type(string) is unicode:
            return str(string)
        return string

    def from_bytes(data):
        """
        Converts the given bytes into a string
        """
        if type(data) is str:
            return data
        return str(data)
else:
    # Python 3
    # pylint: disable=E1101
    STRING_TYPES = (
        bytes,
        str
    )

    NUMERIC_TYPES = (
        int,
        float
    )

    def to_bytes(string):
        """
        Converts the given string into bytes
        """
        if type(string) is bytes:
            return string
        return bytes(string, "UTF-8")

    def from_bytes(data):
        """
        Converts the given bytes into a string
        """
        if type(data) is str:
            return data
        return str(data, "UTF-8")

# ------------------------------------------------------------------------------
# Enumerations

try:
    import enum

    def is_enum(obj):
        """
        Checks if an object is from an enumeration class

        :param obj: Object to test
        :return: True if the object is an enumeration item
        """
        return isinstance(obj, enum.Enum)
except ImportError:
    # Pre-Python 3.4
    def is_enum(_):
        """
        Before Python 3.4, enumerations didn't exist.

        :param _: Object to test
        :return: Always False
        """
        return False

# ------------------------------------------------------------------------------
# Common

DictType = dict

ListType = list
TupleType = tuple

ITERABLE_TYPES = (
    list,
    set, frozenset,
    tuple
)

VALUE_TYPES = (
    bool,
    type(None)
)

PRIMITIVE_TYPES = STRING_TYPES + NUMERIC_TYPES + VALUE_TYPES



import os
import subprocess

from sickchill import logger, settings


class Notifier(object):
    def notify_snatch(self, ep_name):
        pass

    def notify_download(self, ep_name):
        pass

    def notify_subtitle_download(self, ep_name, lang):
        pass

    def notify_git_update(self, new_version):
        pass

    def notify_login(self, ipaddress=""):
        pass

    def moveFolder(self, old_path, new_path):
        self.moveObject(old_path, new_path)

    def moveFile(self, old_file, new_file):
        self.moveObject(old_file, new_file)

    @staticmethod
    def moveObject(old_path, new_path):
        if settings.USE_SYNOINDEX:
            synoindex_cmd = ["/usr/syno/bin/synoindex", "-N", os.path.abspath(new_path), os.path.abspath(old_path)]
            logger.debug("Executing command " + str(synoindex_cmd))
            logger.debug("Absolute path to command: " + os.path.abspath(synoindex_cmd[0]))
            try:
                p = subprocess.Popen(synoindex_cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, cwd=settings.DATA_DIR, universal_newlines=True)
                out, err = p.communicate()
                logger.debug(_("Script result: {0}").format(str(out or err).strip()))
            except OSError as e:
                logger.exception("Unable to run synoindex: " + str(e))

    def deleteFolder(self, cur_path):
        self.makeObject("-D", cur_path)

    def addFolder(self, cur_path):
        self.makeObject("-A", cur_path)

    def deleteFile(self, cur_file):
        self.makeObject("-d", cur_file)

    def addFile(self, cur_file):
        self.makeObject("-a", cur_file)

    @staticmethod
    def makeObject(cmd_arg, cur_path):
        if settings.USE_SYNOINDEX:
            synoindex_cmd = ["/usr/syno/bin/synoindex", cmd_arg, os.path.abspath(cur_path)]
            logger.debug("Executing command " + str(synoindex_cmd))
            logger.debug("Absolute path to command: " + os.path.abspath(synoindex_cmd[0]))
            try:
                p = subprocess.Popen(synoindex_cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, cwd=settings.DATA_DIR, universal_newlines=True)
                out, err = p.communicate()
                logger.debug(_("Script result: {0}").format(str(out or err).strip()))
            except OSError as e:
                logger.exception("Unable to run synoindex: " + str(e))



# -*- coding: utf-8 -*-
from babelfish import LanguageReverseConverter

from ..exceptions import ConfigurationError


class ShooterConverter(LanguageReverseConverter):
    def __init__(self):
        self.from_shooter = {'chn': ('zho',), 'eng': ('eng',)}
        self.to_shooter = {v: k for k, v in self.from_shooter.items()}
        self.codes = set(self.from_shooter.keys())

    def convert(self, alpha3, country=None, script=None):
        if (alpha3,) in self.to_shooter:
            return self.to_shooter[(alpha3,)]

        raise ConfigurationError('Unsupported language for shooter: %s, %s, %s' % (alpha3, country, script))

    def reverse(self, shooter):
        if shooter in self.from_shooter:
            return self.from_shooter[shooter]

        raise ConfigurationError('Unsupported language code for shooter: %s' % shooter)



# -*- coding: utf-8 -*-
import io
import json
import logging
from zipfile import ZipFile

from babelfish import Language
from guessit import guessit
from requests import Session
from six.moves import urllib

from . import Provider
from ..cache import EPISODE_EXPIRATION_TIME, region
from ..exceptions import ProviderError
from ..matches import guess_matches
from ..subtitle import Subtitle, fix_line_ending
from ..video import Episode

logger = logging.getLogger(__name__)


class ArgenteamSubtitle(Subtitle):
    provider_name = 'argenteam'

    def __init__(self, language, download_link, series, season, episode, release, version):
        super(ArgenteamSubtitle, self).__init__(language, download_link)
        self.download_link = download_link
        self.series = series
        self.season = season
        self.episode = episode
        self.release = release
        self.version = version

    @property
    def id(self):
        return self.download_link

    @property
    def info(self):
        return urllib.parse.unquote(self.download_link.rsplit('/')[-1])

    def get_matches(self, video):
        matches = guess_matches(video, {
            'title': self.series,
            'season': self.season,
            'episode': self.episode,
            'release_group': self.version
        })

        # resolution
        if video.resolution and self.version and video.resolution in self.version.lower():
            matches.add('resolution')

        matches |= guess_matches(video, guessit(self.version, {'type': 'episode'}), partial=True)
        return matches


class ArgenteamProvider(Provider):
    provider_name = 'argenteam'
    language = Language.fromalpha2('es')
    languages = {language}
    video_types = (Episode,)
    server_url = "http://argenteam.net/api/v1/"
    subtitle_class = ArgenteamSubtitle

    def __init__(self):
        self.session = None

    def initialize(self):
        self.session = Session()
        self.session.headers['User-Agent'] = self.user_agent

    def terminate(self):
        self.session.close()

    @region.cache_on_arguments(expiration_time=EPISODE_EXPIRATION_TIME, should_cache_fn=lambda value: value)
    def search_episode_id(self, series, season, episode):
        """Search the episode id from the `series`, `season` and `episode`.

        :param str series: series of the episode.
        :param int season: season of the episode.
        :param int episode: episode number.
        :return: the episode id, if any.
        :rtype: int or None

        """
        # make the search
        query = '%s S%#02dE%#02d' % (series, season, episode)
        logger.info('Searching episode id for %r', query)
        r = self.session.get(self.server_url + 'search', params={'q': query}, timeout=10)
        r.raise_for_status()
        results = json.loads(r.text)
        if results['total'] == 1:
            return results['results'][0]['id']

        logger.error('No episode id found for %r', series)

    def query(self, series, season, episode):
        episode_id = self.search_episode_id(series, season, episode)
        if episode_id is None:
            return []

        response = self.session.get(self.server_url + 'episode', params={'id': episode_id}, timeout=10)
        response.raise_for_status()
        content = json.loads(response.text)
        subtitles = []
        for r in content['releases']:
            for s in r['subtitles']:
                subtitle = self.subtitle_class(self.language, s['uri'], series, season, episode, r['team'], r['tags'])
                logger.debug('Found subtitle %r', subtitle)
                subtitles.append(subtitle)

        return subtitles

    def list_subtitles(self, video, languages):
        titles = [video.series] + video.alternative_series
        for title in titles:
            subs = self.query(title, video.season, video.episode)
            if subs:
                return subs

        return []

    def download_subtitle(self, subtitle):
        # download as a zip
        logger.info('Downloading subtitle %r', subtitle)
        r = self.session.get(subtitle.download_link, timeout=10)
        r.raise_for_status()

        # open the zip
        with ZipFile(io.BytesIO(r.content)) as zf:
            if len(zf.namelist()) > 1:
                raise ProviderError('More than one file to unzip')

            subtitle.content = fix_line_ending(zf.read(zf.namelist()[0]))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2017-8-30

@author: generated by @lolobosse script
'''
LOCALE = [
    ["fa un moment", "d'aquí un moment"],
    ["fa %s segons", "d'aquí %s segons"],
    ["fa 1 minut", "d'aquí 1 minut"],
    ["fa %s minuts", "d'aquí %s minuts"],
    ["fa 1 hora", "d'aquí 1 hora"],
    ["fa %s hores", "d'aquí %s hores"],
    ["fa 1 dia", "d'aquí 1 dia"],
    ["fa %s dies", "d'aquí %s dies"],
    ["fa 1 setmana", "d'aquí 1 setmana"],
    ["fa %s setmanes", "d'aquí %s setmanes"],
    ["fa 1 mes", "d'aquí 1 mes"],
    ["fa %s mesos", "d'aquí %s mesos"],
    ["fa 1 any", "d'aquí 1 any"],
    ["fa %s anys", "d'aquí %s anys"]
]
        


import argparse

from stevedore import driver


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument(
        'format',
        nargs='?',
        default='simple',
        help='the output format',
    )
    parser.add_argument(
        '--width',
        default=60,
        type=int,
        help='maximum output width for text',
    )
    parsed_args = parser.parse_args()

    data = {
        'a': 'A',
        'b': 'B',
        'long': 'word ' * 80,
    }

    mgr = driver.DriverManager(
        namespace='stevedore.example.formatter',
        name=parsed_args.format,
        invoke_on_load=True,
        invoke_args=(parsed_args.width,),
    )
    for chunk in mgr.driver.format(data):
        print(chunk, end='')



import datetime
import os
import re
from xml.etree import ElementTree

import sickchill
from sickchill import logger, settings
from sickchill.helper.common import dateFormat, replace_extension
from sickchill.oldbeard import helpers

from . import generic


class MediaBrowserMetadata(generic.GenericMetadata):
    """
    Metadata generation class for Media Browser 2.x/3.x - Standard Mode.

    The following file structure is used:

    show_root/series.xml                       (show metadata)
    show_root/folder.jpg                       (poster)
    show_root/backdrop.jpg                     (fanart)
    show_root/Season ##/folder.jpg             (season thumb)
    show_root/Season ##/filename.ext           (*)
    show_root/Season ##/metadata/filename.xml  (episode metadata)
    show_root/Season ##/metadata/filename.jpg  (episode thumb)
    """

    def __init__(
        self,
        show_metadata=False,
        episode_metadata=False,
        fanart=False,
        poster=False,
        banner=False,
        episode_thumbnails=False,
        season_posters=False,
        season_banners=False,
        season_all_poster=False,
        season_all_banner=False,
    ):

        super().__init__(
            show_metadata, episode_metadata, fanart, poster, banner, episode_thumbnails, season_posters, season_banners, season_all_poster, season_all_banner
        )

        self.name = "MediaBrowser"

        self._ep_nfo_extension = "xml"
        self._show_metadata_filename = "series.xml"

        self.fanart_name = "backdrop.jpg"
        self.poster_name = "folder.jpg"

        # web-ui metadata template
        self.eg_show_metadata = "series.xml"
        self.eg_episode_metadata = "Season##\\metadata\\<i>filename</i>.xml"
        self.eg_fanart = "backdrop.jpg"
        self.eg_poster = "folder.jpg"
        self.eg_banner = "banner.jpg"
        self.eg_episode_thumbnails = "Season##\\metadata\\<i>filename</i>.jpg"
        self.eg_season_posters = "Season##\\folder.jpg"
        self.eg_season_banners = "Season##\\banner.jpg"
        self.eg_season_all_poster = "<i>not supported</i>"
        self.eg_season_all_banner = "<i>not supported</i>"

    # Override with empty methods for unsupported features
    def retrieveShowMetadata(self, folder):
        # while show metadata is generated, it is not supported for our lookup
        return None, None, None

    def create_season_all_poster(self, show_obj):
        pass

    def create_season_all_banner(self, show_obj):
        pass

    def get_episode_file_path(self, ep_obj):
        """
        Returns a full show dir/metadata/episode.xml path for MediaBrowser
        episode metadata files

        ep_obj: a TVEpisode object to get the path for
        """

        if os.path.isfile(ep_obj.location):
            xml_filename = replace_extension(os.path.basename(ep_obj.location), self._ep_nfo_extension)
            metadata_dir_name = os.path.join(os.path.dirname(ep_obj.location), "metadata")
            xml_file_path = os.path.join(metadata_dir_name, xml_filename)
        else:
            logger.debug("Episode location doesn't exist: " + str(ep_obj.location))
            return ""

        return xml_file_path

    @staticmethod
    def get_episode_thumb_path(ep_obj):
        """
        Returns a full show dir/metadata/episode.jpg path for MediaBrowser
        episode thumbs.

        ep_obj: a TVEpisode object to get the path from
        """

        if os.path.isfile(ep_obj.location):
            tbn_filename = replace_extension(os.path.basename(ep_obj.location), "jpg")
            metadata_dir_name = os.path.join(os.path.dirname(ep_obj.location), "metadata")
            tbn_file_path = os.path.join(metadata_dir_name, tbn_filename)
        else:
            return None

        return tbn_file_path

    @staticmethod
    def get_season_poster_path(show_obj, season):
        """
        Season thumbs for MediaBrowser go in Show Dir/Season X/folder.jpg

        If no season folder exists, None is returned
        """

        dir_list = [x for x in os.listdir(show_obj.location) if os.path.isdir(os.path.join(show_obj.location, x))]

        season_dir_regex = r"^Season\s+(\d+)$"

        season_dir = None

        for cur_dir in dir_list:
            # MediaBrowser 1.x only supports 'Specials'
            # MediaBrowser 2.x looks to only support 'Season 0'
            # MediaBrowser 3.x looks to mimic KODI/Plex support
            if season == 0 and cur_dir == "Specials":
                season_dir = cur_dir
                break

            match = re.match(season_dir_regex, cur_dir, re.I)
            if not match:
                continue

            cur_season = int(match.group(1))

            if cur_season == season:
                season_dir = cur_dir
                break

        if not season_dir:
            logger.debug("Unable to find a season dir for season " + str(season))
            return None

        logger.debug("Using " + str(season_dir) + "/folder.jpg as season dir for season " + str(season))

        return os.path.join(show_obj.location, season_dir, "folder.jpg")

    @staticmethod
    def get_season_banner_path(show_obj, season):
        """
        Season thumbs for MediaBrowser go in Show Dir/Season X/banner.jpg

        If no season folder exists, None is returned
        """

        dir_list = [x for x in os.listdir(show_obj.location) if os.path.isdir(os.path.join(show_obj.location, x))]

        season_dir_regex = r"^Season\s+(\d+)$"

        season_dir = None

        for cur_dir in dir_list:
            # MediaBrowser 1.x only supports 'Specials'
            # MediaBrowser 2.x looks to only support 'Season 0'
            # MediaBrowser 3.x looks to mimic KODI/Plex support
            if season == 0 and cur_dir == "Specials":
                season_dir = cur_dir
                break

            match = re.match(season_dir_regex, cur_dir, re.I)
            if not match:
                continue

            cur_season = int(match.group(1))

            if cur_season == season:
                season_dir = cur_dir
                break

        if not season_dir:
            logger.debug("Unable to find a season dir for season " + str(season))
            return None

        logger.debug("Using " + str(season_dir) + "/banner.jpg as season dir for season " + str(season))

        return os.path.join(show_obj.location, season_dir, "banner.jpg")

    def _show_data(self, show_obj):
        """
        Creates an elementTree XML structure for a MediaBrowser-style series.xml
        returns the resulting data object.

        show_obj: a TVShow instance to create the NFO for
        """

        tv_node = ElementTree.Element("Series")

        myShow = sickchill.indexer.series(show_obj)
        if not myShow:
            logger.exception("Unable to find show with id {} on {}, skipping it".format(show_obj.indexerid, show_obj.indexer_name))
            return False

        # check for title and id
        if not (getattr(myShow, "seriesName", None) and getattr(myShow, "id", None)):
            logger.info("Incomplete info for show with id {} on {}, skipping it".format(show_obj.indexerid, show_obj.indexer_name))
            return False

        data = getattr(myShow, "id", None)
        if data:
            indexerid = ElementTree.SubElement(tv_node, "id")
            indexerid.text = str(data)

        data = getattr(myShow, "seriesName", None)
        if data:
            SeriesName = ElementTree.SubElement(tv_node, "SeriesName")
            SeriesName.text = data

        data = getattr(myShow, "status", None)
        if data:
            Status = ElementTree.SubElement(tv_node, "Status")
            Status.text = data

        data = getattr(myShow, "network", None)
        if data:
            Network = ElementTree.SubElement(tv_node, "Network")
            Network.text = data

            Studios = ElementTree.SubElement(tv_node, "Studios")
            Studio = ElementTree.SubElement(Studios, "Studio")
            Studio.text = data

        data = getattr(myShow, "airsTime", None)
        if data:
            airsTime = ElementTree.SubElement(tv_node, "Airs_Time")
            airsTime.text = data

        data = getattr(myShow, "airsDayOfWeek", None)
        if data:
            airsDayOfWeek = ElementTree.SubElement(tv_node, "Airs_DayOfWeek")
            airsDayOfWeek.text = data

        data = getattr(myShow, "firstAired", None)
        if data:
            FirstAired = ElementTree.SubElement(tv_node, "FirstAired")
            FirstAired.text = data
            PremiereDate = ElementTree.SubElement(tv_node, "PremiereDate")
            PremiereDate.text = data
            try:
                year_text = str(datetime.datetime.strptime(data, dateFormat).year)
                if year_text:
                    ProductionYear = ElementTree.SubElement(tv_node, "ProductionYear")
                    ProductionYear.text = year_text
            except Exception:
                pass

        data = getattr(myShow, "rating", None)
        if data:
            ContentRating = ElementTree.SubElement(tv_node, "ContentRating")
            ContentRating.text = data

            MPAARating = ElementTree.SubElement(tv_node, "MPAARating")
            MPAARating.text = data

            certification = ElementTree.SubElement(tv_node, "certification")
            certification.text = data

            Rating = ElementTree.SubElement(tv_node, "Rating")
            Rating.text = data

        MetadataType = ElementTree.SubElement(tv_node, "Type")
        MetadataType.text = "Series"

        data = getattr(myShow, "overview", None)
        if data:
            Overview = ElementTree.SubElement(tv_node, "Overview")
            Overview.text = data

        data = getattr(myShow, "runtime", None)
        if data:
            RunningTime = ElementTree.SubElement(tv_node, "RunningTime")
            RunningTime.text = data

            Runtime = ElementTree.SubElement(tv_node, "Runtime")
            Runtime.text = data

        data = getattr(myShow, "imdbId", None)
        if data:
            imdb_id = ElementTree.SubElement(tv_node, "IMDB_ID")
            imdb_id.text = data

            imdb_id = ElementTree.SubElement(tv_node, "IMDB")
            imdb_id.text = data

            imdb_id = ElementTree.SubElement(tv_node, "IMDbId")
            imdb_id.text = data

        data = getattr(myShow, "zap2itId", None)
        if data:
            Zap2ItId = ElementTree.SubElement(tv_node, "Zap2ItId")
            Zap2ItId.text = data

        if getattr(myShow, "genre", []) and isinstance(myShow.genre, list):
            Genres = ElementTree.SubElement(tv_node, "Genres")
            for genre in myShow.genre:
                if genre.strip():
                    cur_genre = ElementTree.SubElement(Genres, "Genre")
                    cur_genre.text = genre.strip()

            Genre = ElementTree.SubElement(tv_node, "Genre")
            Genre.text = "|".join(myShow.genre)

        helpers.indentXML(tv_node)

        return ElementTree.ElementTree(tv_node)

    def _ep_data(self, ep_obj):
        """
        Creates an elementTree XML structure for a MediaBrowser style episode.xml
        and returns the resulting data object.

        show_obj: a TVShow instance to create the NFO for
        """

        eps_to_write = [ep_obj] + ep_obj.relatedEps

        persons_dict = {"Director": [], "GuestStar": [], "Writer": []}

        myShow = ep_obj.idxr.series_from_episode(ep_obj)
        if not myShow:
            logger.info("Unable to connect to {} while creating meta files - skipping".format(ep_obj.idxr.name))
            return False

        rootNode = ElementTree.Element("Item")

        # write an MediaBrowser XML containing info for all matching episodes
        for curEpToWrite in eps_to_write:

            myEp = curEpToWrite.idxr.episode(curEpToWrite)
            if not myEp:
                continue

            if curEpToWrite == ep_obj:
                # root (or single) episode

                # default to today's date for specials if firstAired is not set
                if ep_obj.airdate != datetime.date.min and myEp.get("firstAired"):
                    myEp["firstAired"] = str(ep_obj.airdate)

                if ep_obj.season == 0 and not myEp.get("firstAired"):
                    if ep_obj.show and ep_obj.show.startyear:
                        myEp["firstAired"] = str(datetime.date.min.replace(year=ep_obj.show.startyear))
                    else:
                        myEp["firstAired"] = str(datetime.date.today())

                if not (myEp.get("episodeName") and myEp.get("firstAired")):
                    return None

                episode = rootNode

                if curEpToWrite.name:
                    EpisodeName = ElementTree.SubElement(episode, "EpisodeName")
                    EpisodeName.text = curEpToWrite.name

                EpisodeNumber = ElementTree.SubElement(episode, "EpisodeNumber")
                EpisodeNumber.text = str(ep_obj.episode)

                if ep_obj.relatedEps:
                    try:
                        max_episode = max(e.episode for e in ep_obj.relatedEps if e)
                    except (AttributeError, TypeError):
                        max_episode = curEpToWrite.episode

                    EpisodeNumberEnd = ElementTree.SubElement(episode, "EpisodeNumberEnd")
                    EpisodeNumberEnd.text = str(max_episode)

                SeasonNumber = ElementTree.SubElement(episode, "SeasonNumber")
                SeasonNumber.text = str(curEpToWrite.season)

                if myEp.get("absoluteNumber") and not ep_obj.relatedEps:
                    absolute_number = ElementTree.SubElement(episode, "absolute_number")
                    absolute_number.text = str(myEp["absoluteNumber"])

                if curEpToWrite.airdate != datetime.date.min:
                    FirstAired = ElementTree.SubElement(episode, "FirstAired")
                    FirstAired.text = str(curEpToWrite.airdate)

                MetadataType = ElementTree.SubElement(episode, "Type")
                MetadataType.text = "Episode"

                if curEpToWrite.description:
                    Overview = ElementTree.SubElement(episode, "Overview")
                    Overview.text = curEpToWrite.description
                elif myEp.get("overview"):
                    Overview = ElementTree.SubElement(episode, "Overview")
                    Overview.text = myEp["overview"]

                if not ep_obj.relatedEps:
                    if myEp.get("rating"):
                        Rating = ElementTree.SubElement(episode, "Rating")
                        Rating.text = myEp["siteRating"]

                    if myEp.get("imdbId"):
                        IMDB_ID = ElementTree.SubElement(episode, "IMDB_ID")
                        IMDB_ID.text = myEp["imdbId"]

                        IMDB = ElementTree.SubElement(episode, "IMDB")
                        IMDB.text = myEp["imdbId"]

                        IMDbId = ElementTree.SubElement(episode, "IMDbId")
                        IMDbId.text = myEp["imdbId"]

                if myEp.get("id"):
                    indexerid = ElementTree.SubElement(episode, "id")
                    indexerid.text = str(myEp["id"])

                Persons = ElementTree.SubElement(episode, "Persons")

                for actor in ep_obj.idxr.actors(myShow):
                    if not ("name" in actor and actor["name"].strip()):
                        continue

                    cur_actor = ElementTree.SubElement(Persons, "Person")

                    cur_actor_name = ElementTree.SubElement(cur_actor, "Name")
                    cur_actor_name.text = actor["name"].strip()

                    cur_actor_type = ElementTree.SubElement(cur_actor, "Type")
                    cur_actor_type.text = "Actor"

                    if "role" in actor and actor["role"].strip():
                        cur_actor_role = ElementTree.SubElement(cur_actor, "Role")
                        cur_actor_role.text = actor["role"].strip()

                if myEp.get("language"):
                    language = myEp.get("language")["overview"]
                else:
                    language = settings.INDEXER_DEFAULT_LANGUAGE

                Language = ElementTree.SubElement(episode, "Language")
                Language.text = language

                if myEp.get("filename"):
                    thumb = ElementTree.SubElement(episode, "filename")
                    thumb.text = curEpToWrite.idxr.complete_image_url(myEp["filename"])

            else:
                # append data from (if any) related episodes
                if curEpToWrite.name:
                    if not EpisodeName.text:
                        EpisodeName.text = curEpToWrite.name
                    else:
                        EpisodeName.text = EpisodeName.text + ", " + curEpToWrite.name

                if curEpToWrite.description:
                    if not Overview.text:
                        Overview.text = curEpToWrite.description
                    else:
                        Overview.text = Overview.text + "\r" + curEpToWrite.description

            # collect all directors, guest stars and writers
            if myEp.get("directors"):
                persons_dict["Director"] += myEp["directors"]
            if myEp.get("guestStars"):
                persons_dict["GuestStar"] += myEp["guestStars"]
            if myEp.get("writers"):
                persons_dict["Writer"] += myEp["writers"]

        # fill in Persons section with collected directors, guest starts and writers
        for person_type, names in persons_dict.items():
            # remove doubles
            names = list(set(names))
            for cur_name in names:
                Person = ElementTree.SubElement(Persons, "Person")
                cur_person_name = ElementTree.SubElement(Person, "Name")
                cur_person_name.text = cur_name
                cur_person_type = ElementTree.SubElement(Person, "Type")
                cur_person_type.text = person_type

        helpers.indentXML(rootNode)

        return ElementTree.ElementTree(rootNode)


# present a standard "interface" from the module
metadata_class = MediaBrowserMetadata



from unittest import TestCase
from fanart.immutable import Immutable


class TestMutable(object):
    def __init__(self, spam, ham, eggs):
        self.spam = spam
        self.ham = ham
        self.eggs = eggs

    @Immutable.mutablemethod
    def anyway(self):
        self.spam = self.ham + self.eggs


class TestImmutable(TestMutable, Immutable):
    @Immutable.mutablemethod
    def __init__(self, *args, **kwargs):
        super(TestImmutable, self).__init__(*args, **kwargs)


class ImmutableTestCase(TestCase):
    def setUp(self):
        self.instance = TestImmutable('spam', 'ham', 'eggs')

    def test_set_raises(self):
        self.assertRaises(TypeError, self.instance.__setattr__, 'spam', 'ham')

    def test_set(self):
        self.instance._mutable = True
        self.instance.spam = 'ham'
        self.assertEqual(self.instance.spam, 'ham')

    def test_del_raises(self):
        self.assertRaises(TypeError, self.instance.__delattr__, 'spam')

    def test_del(self):
        self.instance._mutable = True
        del self.instance.spam
        self.assertRaises(AttributeError, self.instance.__getattribute__, 'spam')

    def test_equal(self):
        new_instance = TestImmutable('spam', 'ham', 'eggs')
        self.assertEqual(self.instance, new_instance)

    def test_mutable_dec(self):
        instance = TestMutable('spam', 'ham', 'eggs')
        instance.anyway()
        self.assertEqual(instance.spam, 'hameggs')



import re

from sickchill import logger
from sickchill.helper.common import convert_size, try_int
from sickchill.oldbeard import tvcache
from sickchill.oldbeard.bs4_parser import BS4Parser
from sickchill.providers.torrent.TorrentProvider import TorrentProvider


class Provider(TorrentProvider):
    def __init__(self):

        super().__init__("TokyoToshokan")

        self.public = True
        self.supports_absolute_numbering = True
        self.anime_only = True

        self.minseed = 0
        self.minleech = 0

        self.url = "http://tokyotosho.info/"
        self.urls = {"search": self.url + "search.php", "rss": self.url + "rss.php"}
        self.cache = tvcache.TVCache(self, min_time=15)  # only poll TokyoToshokan every 15 minutes max

    def search(self, search_strings, age=0, ep_obj=None):
        results = []
        if self.show and not self.show.is_anime:
            return results

        for mode in search_strings:
            items = []
            logger.debug(_("Search Mode: {mode}".format(mode=mode)))
            for search_string in {*search_strings[mode]}:
                if mode != "RSS":
                    logger.debug(_("Search String: {search_string}".format(search_string=search_string)))

                search_params = {
                    "terms": search_string,
                    "type": 1,  # get anime types
                }

                data = self.get_url(self.urls["search"], params=search_params, returns="text")
                if not data:
                    continue

                with BS4Parser(data, "html5lib") as soup:
                    torrent_table = soup.find("table", class_="listing")
                    torrent_rows = torrent_table("tr") if torrent_table else []

                    # Continue only if one Release is found
                    if len(torrent_rows) < 2:
                        logger.debug("Data returned from provider does not contain any torrents")
                        continue

                    a = 1 if len(torrent_rows[0]("td")) < 2 else 0

                    for top, bot in zip(torrent_rows[a::2], torrent_rows[a + 1 :: 2]):
                        try:
                            desc_top = top.find("td", class_="desc-top")
                            title = desc_top.get_text(strip=True)
                            download_url = desc_top.find("a")["href"]

                            desc_bottom = bot.find("td", class_="desc-bot").get_text(strip=True)
                            size = convert_size(desc_bottom.split("|")[1].strip("Size: ")) or -1

                            stats = bot.find("td", class_="stats").get_text(strip=True)
                            sl = re.match(r"S:(?P<seeders>\d+)L:(?P<leechers>\d+)C:(?:\d+)ID:(?:\d+)", stats.replace(" ", ""))
                            seeders = try_int(sl.group("seeders")) if sl else 0
                            leechers = try_int(sl.group("leechers")) if sl else 0
                        except Exception:
                            continue

                        if not all([title, download_url]):
                            continue

                        # Filter unseeded torrent
                        if seeders < self.minseed or leechers < self.minleech:
                            if mode != "RSS":
                                logger.debug(
                                    "Discarding torrent because it doesn't meet the minimum seeders or leechers: {0} (S:{1} L:{2})".format(
                                        title, seeders, leechers
                                    )
                                )
                            continue

                        item = {"title": title, "link": download_url, "size": size, "seeders": seeders, "leechers": leechers, "hash": ""}
                        if mode != "RSS":
                            logger.debug("Found result: {0} with {1} seeders and {2} leechers".format(title, seeders, leechers))

                        items.append(item)

            # For each search mode sort all the items by seeders if available
            items.sort(key=lambda d: try_int(d.get("seeders", 0)), reverse=True)
            results += items

        return results



# -*- coding: utf-8 -*-

"""
This module implements the Series functionality of TheTVDb API.
Allows to retrieve series info, actors, basic episodes info and images.

See [Series API section](https://api.thetvdb.com/swagger#/Series)
"""

from .base import TVDB

class Series(TVDB):
    """
    Series class to retrieve all the info about a series.
    Requires the series id.
    """
    _BASE_PATH = 'series'
    _URLS = {
        'info': '/{id}',
        'actors': '/{id}/actors'
    }	

    def __init__(self, id, language=''):
        """
        Initialize the series class.
        `id` is the TheTVDb series id. You can also provide `language`, 
        the language id you want to use to retrieve the info.
        """
        self._set_language(language)
        self.Images = Series_Images(id, language)
        """
        Allows to retrieve images info.
        """
        self.Episodes = Series_Episodes(id, language)
        """
        Allows to retrieve episodes info.
        """
        super(Series, self).__init__(id)

    def info(self, language=''):
        """
        Get the basic show information for a specific show id and sets 
        them to the attributes.

        You also provide `language`, the language id you want to use 
        to retrieve the info.

        Returns a dict with the series info

        For example

            #!python
            >>> import tvdbsimple as tvdb
            >>> tvdb.KEYS.API_KEY = 'YOUR_API_KEY'
            >>> show = tvdb.Series(78804)
            >>> response = show.info()
            >>> show.seriesName
            'Doctor Who (2005)'

        """
        path = self._get_id_path('info')
        
        self._set_language(language)
        response = self._GET(path)
        self._set_attrs_to_values(response)
        return response

    def actors(self, language=''):
        """
        Get the actors for the show id and sets them to 
        `actors` the attributes.

        You also provide `language`, the language id you want to use 
        to retrieve the info.

        Returns a list of actors with their info.

        For example

            #!python
            >>> import tvdbsimple as tvdb
            >>> tvdb.KEYS.API_KEY = 'YOUR_API_KEY'
            >>> show = tvdb.Series(78804)
            >>> response = show.actors()
            >>> show.actors[0]['name']
            u'David Tennant'

        """
        path = self._get_id_path('actors')
        
        self._set_language(language)
        response = self._GET(path)
        self._set_attrs_to_values({'actors': response})
        return response

class Series_Episodes(TVDB):
    """
    Class needed to organize series episodes. Allow to retrieve basic 
    info episodes for the series.
    Requires the series id.
    """
    _BASE_PATH = 'series'
    _URLS = {
        'summary': '/{id}/episodes/summary',
        'episodes': '/{id}/episodes',
        'query': '/{id}/episodes/query',
        'query_params': '/{id}/episodes/query/params'
    }

    def __init__(self, id, language='', **kwargs):
        """
        Initialize the class.

        `id` is the TheTVDb series id.
        You can  provide `language`, the language id you want to use to 
        retrieve the info.
        You can  provide `absoluteNumber` to get only episodes with the 
        provided absolute number.
        You can  provide `airedSeason` to get only episodes with the 
        provided aired season number.
        You can  provide `airedEpisode` to get only episodes with the 
        provided aired episode number.
        You can  provide `dvdSeason` to get only episodes with the 
        provided dvd season number.
        You can  provide `dvdEpisode` to get only episodes with the 
        provided dvd episode number.
        You can  provide `imdbId` to get only episodes with the 
        provided imdb id.
        """
        super(Series_Episodes, self).__init__(id)
        self._set_language(language)
        self._FILTERS = {}
        self.update_filters(**kwargs)
        self._PAGES = -1
        self._PAGES_LIST = {}

    def update_filters(self, **kwargs):
        """
        Set the filters for the episodes of the specific show id.

        You can  provide `absoluteNumber` to get only episodes with the 
        provided absolute number.
        You can  provide `airedSeason` to get only episodes with the 
        provided aired season number.
        You can  provide `airedEpisode` to get only episodes with the 
        provided aired episode number.
        You can  provide `dvdSeason` to get only episodes with the 
        provided dvd season number.
        You can  provide `dvdEpisode` to get only episodes with the 
        provided dvd episode number.
        You can  provide `imdbId` to get only episodes with the 
        provided imdb id.
        """
        self._PAGES = -1
        self._PAGES_LIST = {}
        self._FILTERS=kwargs

    def summary(self):
        """
        Get the episodes summary for a specific show id and sets 
        them to their attributes.

        Returns a dict with a summary of the episodes.

        For example

            #!python
            >>> import tvdbsimple as tvdb
            >>> tvdb.KEYS.API_KEY = 'YOUR_API_KEY'
            >>> showeps = tvdb.Series_Episodes(78804)
            >>> response = showeps.summary()
            >>> showeps.airedEpisodes
            '267'

        """
        path = self._get_id_path('summary')
        
        response = self._GET(path)
        self._set_attrs_to_values(response)
        return response

    def query_params(self):
        """
        Get the query parameters allowed for a specific show id.

        Returns a dict with all the filters allowed.
        """
        path = self._get_id_path('query_params')
        
        response = self._GET(path)
        self._set_attrs_to_values({'query_params': response})
        return response

    def pages(self):
        """
        Get the number of episode pages for filtered episodes of a specific show id.

        Returns the integer number of pages for filtered episodes of the show.
        """
        if self._PAGES < 0:
            self.page(1)
        return self._PAGES

    def all(self):
        """
        Get the full episode list with basic details for a specific show id.
        
        Returns a list of episodes with basic info.

        For example

            #!python
            >>> import tvdbsimple as tvdb
            >>> tvdb.KEYS.API_KEY = 'YOUR_API_KEY'
            >>> showeps = tvdb.Series_Episodes(78804)
            >>> response = showeps.all()
            >>> showeps.episodes[0]['episodeName']
            'Rose'

        """
        episodes = []
        for i in range (1, self.pages()+1):
            episodes.extend(self.page(i))
        
        self._set_attrs_to_values({'episodes': episodes})
        return episodes

    def page(self, page):
        """
        Get the episode list for a specific page of a show id.
        `page` is the page number of the episodes list to retrieve

        Returns a list episodes with basic info.
        """
        if page in self._PAGES_LIST:
            return self._PAGES_LIST[page]
        if self._FILTERS:
            path = self._get_id_path('query')
        else:
            path = self._get_id_path('episodes')
        
        filters = self._FILTERS.copy()
        filters['page'] = page
        response = self._GET(path, params=filters, cleanJson=False)
        if 'links' in response and 'last' in response['links']:
            self._PAGES = response['links']['last']

        self._PAGES_LIST[page] = response['data']
        return response['data']

    def __iter__(self):
        for i in range (1, self.pages()+1):
            yield self.page(i)

class Series_Images(TVDB):
    """
    Class needed to organize series images. Allow to retrieve all 
    the types of images of a series.
    Requires the series id.

        language: (optional) language to request.
        resolution: (optional) image resolution.
        subKey: (optional) subkey research.
    """
    _BASE_PATH = 'series'
    _URLS = {
        'imagequery': '/{id}/images/query',
        'summary': '/{id}/images',
        'query_params': '/{id}/images/query/params'
    }

    def __init__(self, id, language='', **kwargs):
        """
        Initialize the class.

        `id` is the TheTVDb series id.
        You can  provide `language`, the language id you want to use to 
        retrieve the info.
        You can  provide `reqolution` to get only episodes with the 
        provided resolution.
        You can  provide `subKey` to get only episodes with the 
        provided subKey.
        """
        super(Series_Images, self).__init__(id)
        self._set_language(language)
        self.update_filters(**kwargs)

    def update_filters(self, **kwargs):
        """
        Set the filters for the episodes of the specific show id.

        You can  provide `language`, the language id you want to use to 
        retrieve the info.
        You can  provide `reqolution` to get only episodes with the 
        provided resolution.
        You can  provide `subKey` to get only episodes with the 
        provided subKey.
        """
        self._FILTERS=kwargs

    def summary(self):
        """
        Get the images summary for a specific show id and sets 
        them to `summary` attributes.

        Returns a dict with a summary of the images.

        For example

            #!python
            >>> import tvdbsimple as tvdb
            >>> tvdb.KEYS.API_KEY = 'YOUR_API_KEY'
            >>> showimgs = tvdb.Series_Images(78804)
            >>> response = showimgs.summary()
            >>> showimgs.summary['poster']
            53

        """
        path = self._get_id_path('summary')
        
        response = self._GET(path)
        self._set_attrs_to_values(response)
        return response

    def query_params(self):
        """
        Get the query parameters allowed for a specific show id.

        Returns:
            A dict respresentation of the JSON returned from the API.
        """
        path = self._get_id_path('query_params')
        
        response = self._GET(path)
        self._set_attrs_to_values({'query_params': response})
        return response


    def poster(self, language=''):
        """
        Get the posters for a specific show.
        
        You can  provide `language`, the language id you want to use to 
        retrieve the info.

        Returns a list of posters.

        For example

            #!python
            >>> import tvdbsimple as tvdb
            >>> tvdb.KEYS.API_KEY = 'YOUR_API_KEY'
            >>> showimgs = tvdb.Series_Images(78804)
            >>> response = showimgs.poster()
            >>> showimgs.poster[0]['resolution']
            '680x1000'

        """
        return self._get_image_type('poster', language)

    def fanart(self, language=''):
        """
        Get the fanarts for a specific show.
        
        You can  provide `language`, the language id you want to use to 
        retrieve the info.

        Returns a list of fanarts.

        For example

            #!python
            >>> import tvdbsimple as tvdb
            >>> tvdb.KEYS.API_KEY = 'YOUR_API_KEY'
            >>> showimgs = tvdb.Series_Images(78804)
            >>> response = showimgs.fanart()
            >>> showimgs.fanart[0]['resolution']
            '1280x720'

        """
        return self._get_image_type('fanart', language)

    def series(self, language=''):
        """
        Get the series images for a specific show.
        
        You can  provide `language`, the language id you want to use to 
        retrieve the info.

        Returns a list of series images.

        For example

            #!python
            >>> import tvdbsimple as tvdb
            >>> tvdb.KEYS.API_KEY = 'YOUR_API_KEY'
            >>> showimgs = tvdb.Series_Images(78804)
            >>> response = showimgs.series()
            >>> showimgs.series[0]['thumbnail']
            '_cache/text/34391.jpg'

        """
        return self._get_image_type('series', language)

    def season(self, language=''):
        """
        Get the season images for a specific show.
        
        You can  provide `language`, the language id you want to use to 
        retrieve the info.

        Returns a list of season images.

        For example

            #!python
            >>> import tvdbsimple as tvdb
            >>> tvdb.KEYS.API_KEY = 'YOUR_API_KEY'
            >>> showimgs = tvdb.Series_Images(78804)
            >>> response = showimgs.season()
            >>> showimgs.season[0]['thumbnail']
            '_cache/seasons/34391-1.jpg'

        """
        return self._get_image_type('season', language)

    def seasonwide(self, language=''):
        """
        Get the seasonwide images for a specific show.
        
        You can  provide `language`, the language id you want to use to 
        retrieve the info.

        Returns a list of seasonwide images.

        For example

            #!python
            >>> import tvdbsimple as tvdb
            >>> tvdb.KEYS.API_KEY = 'YOUR_API_KEY'
            >>> showimgs = tvdb.Series_Images(78804)
            >>> response = showimgs.seasonwide()
            >>> showimgs.seasonwide[0]['thumbnail']
            '_cache/seasonswide/78804-1.jpg'

        """
        return self._get_image_type('seasonwide', language)

    def all(self, language=''):
        """
        Get all the images for a specific show and sets it to `images` attribute.
        It needs to have at least one filter set.
        
        You can  provide `language`, the language id you want to use to 
        retrieve the info.

        Returns a list of images.

        For example

            #!python
            >>> import tvdbsimple as tvdb
            >>> tvdb.KEYS.API_KEY = 'YOUR_API_KEY'
            >>> showimgs = tvdb.Series_Images(78804, resolution='1280x720')
            >>> response = showimgs.all()
            >>> showimgs.images[0]['resolution']
            '1280x720'

        """
        return self._get_image_type('images', language)

    def _get_image_type(self, type, language=''):
        path = self._get_id_path('imagequery')
        
        self._set_language(language)
        filters = self._FILTERS.copy()
        if type != 'images':
            filters['keyType'] = type

        response = self._GET(path, params=filters)
        self._set_attrs_to_values({type: response})
        return response



import unittest

from sickchill import settings
from sickchill.oldbeard.show_name_helpers import hasPreferredWords


class PreferWordFilterTest(unittest.TestCase):
    def setUp(self):
        settings.PREFER_WORDS = "atmos,7.1,5.1,ddp,dd"

        # prefer words only works on names, so we don't need complete shows
        self.results_names_only_source = [
            "Archer.2009.S01E10.1080p.WEB.x264-MEMENTO",
            "Archer.2009.S01E10.1080p.WEB.dd2.0.x264-MEMENTO",
            "Archer.2009.S10E08.1080p.AMZN.WEB-DL.DDP5.1.H.264-NT",
            "Archer.2009.S10E08.1080p.AMZN.WEB-DL.Atmos.DDP5.1.H.264-NT",
            "Archer.2009.S10E08.1080p.AMZN.WEB-DL.Atmos.DDP7.1.H.264-NT",
            "Archer.2009.S10E08.1080p.AMZN.WEB-DL.True-HD.DDP7.1.H.264-NT",
            "Archer.2009.S10E08.1080p.AMZN.WEB-DL.DDP.H.264-NT",
        ]

    def test_prefer_words_determine_weight(self):
        assert hasPreferredWords(self.results_names_only_source[0]) == 0
        assert hasPreferredWords(self.results_names_only_source[1]) == 1
        assert hasPreferredWords(self.results_names_only_source[2]) == 3
        assert hasPreferredWords(self.results_names_only_source[3]) == 5
        assert hasPreferredWords(self.results_names_only_source[4]) == 5
        assert hasPreferredWords(self.results_names_only_source[5]) == 4
        assert hasPreferredWords(self.results_names_only_source[6]) == 2


if __name__ == "__main__":
    SUITE = unittest.TestLoader().loadTestsFromTestCase(PreferWordFilterTest)
    unittest.TextTestRunner(verbosity=2).run(SUITE)



# -*- coding: utf-8 -*-
import datetime

import six
from dogpile.cache import make_region
from dogpile.cache.util import function_key_generator

#: Expiration time for show caching
SHOW_EXPIRATION_TIME = datetime.timedelta(weeks=3).total_seconds()

#: Expiration time for episode caching
EPISODE_EXPIRATION_TIME = datetime.timedelta(days=3).total_seconds()

#: Expiration time for scraper searches
REFINER_EXPIRATION_TIME = datetime.timedelta(weeks=1).total_seconds()


def _to_native_str(value):
    if six.PY2:
        # In Python 2, the native string type is bytes
        if isinstance(value, six.text_type):  # unicode for Python 2
            return value.encode('utf-8')
        else:
            return six.binary_type(value)
    else:
        # In Python 3, the native string type is unicode
        if isinstance(value, six.binary_type):  # bytes for Python 3
            return value.decode('utf-8')
        else:
            return six.text_type(value)


def to_native_str_key_generator(namespace, fn, to_str=_to_native_str):
    return function_key_generator(namespace, fn, to_str)


region = make_region(function_key_generator=to_native_str_key_generator)



#!/usr/bin/env python3
import re
import sys

import toml
from setuptools import setup

with open("pyproject.toml", "r") as f:
    pyproject = toml.loads(f.read())

config = pyproject["tool"]["poetry"]
prod = config["dependencies"]
dev = config["dev-dependencies"]

author = None
author_email = None
def parse_authors():
    global author
    global author_email

    parsed = re.match(r"(?P<name>.*) <(?P<email>.*)>", config["authors"][0])
    author, author_email = parsed.groups()

if "setup.py" in sys.argv[0] or 'poetry' in sys.argv[0]:
    parse_authors()
    setup(
        name=config["name"],
        packages=["sickchill"],
        version=config["version"],
        description=config["description"],
        author=author,
        author_email=author_email,
        license=config["license"],
        url=config["homepage"],
        download_url=config["repository"],
        classifiers=config["classifiers"],
        install_requires=[item + prod[item].replace("^", ">=") if prod[item] != "*" else item for item in prod if item != "python"],
        extras_require={"dev": [item + dev[item].replace("^", ">=") if dev[item] != "*" else item for item in dev if item != "python"]},
        message_extractors={
            "gui": [
                ("**/views/**.mako", "mako", {"input_encoding": "utf-8"}),
                ("**/js/*.min.js", "ignore", None),
                ("**/js/*.js", "javascript", {"input_encoding": "utf-8"})
            ],
            "sickchill": [("**.py", "python", None)],
        },
        include_package_data=True,
    )



from urllib.parse import urljoin

import validators

from sickchill import logger
from sickchill.helper.common import convert_size, try_int
from sickchill.oldbeard import tvcache
from sickchill.providers.torrent.TorrentProvider import TorrentProvider


class Provider(TorrentProvider):
    def __init__(self):

        super().__init__("BitCannon")

        self.minseed = 0
        self.minleech = 0
        self.custom_url = None
        self.api_key = None

        self.cache = tvcache.TVCache(self, search_params={"RSS": ["tv", "anime"]})

    def search(self, search_strings, age=0, ep_obj=None):
        results = []

        url = "http://localhost:3000/"
        if self.custom_url:
            if not validators.url(self.custom_url):
                logger.warning("Invalid custom url set, please check your settings")
                return results
            url = self.custom_url

        search_params = {}

        anime = ep_obj and ep_obj.show and ep_obj.show.anime
        search_params["category"] = ("tv", "anime")[bool(anime)]

        if self.api_key:
            search_params["apiKey"] = self.api_key

        for mode in search_strings:
            items = []
            logger.debug(_("Search Mode: {mode}".format(mode=mode)))
            for search_string in {*search_strings[mode]}:
                search_params["q"] = search_string
                if mode != "RSS":
                    logger.debug("Search string: {0}".format(search_string))

                search_url = urljoin(url, "api/search")
                parsed_json = self.get_url(search_url, params=search_params, returns="json")
                if not parsed_json:
                    logger.debug("No data returned from provider")
                    continue

                if not self._check_auth_from_data(parsed_json):
                    return results

                for result in parsed_json.pop("torrents", {}):
                    try:
                        title = result.pop("title", "")

                        info_hash = result.pop("infoHash", "")
                        download_url = "magnet:?xt=urn:btih:" + info_hash
                        if not all([title, download_url, info_hash]):
                            continue

                        swarm = result.pop("swarm", None)
                        if swarm:
                            seeders = try_int(swarm.pop("seeders", 0))
                            leechers = try_int(swarm.pop("leechers", 0))
                        else:
                            seeders = leechers = 0

                        if seeders < self.minseed or leechers < self.minleech:
                            if mode != "RSS":
                                logger.debug(
                                    "Discarding torrent because it doesn't meet the "
                                    "minimum seeders or leechers: {0} (S:{1} L:{2})".format(title, seeders, leechers)
                                )
                            continue

                        size = convert_size(result.pop("size", -1)) or -1
                        item = {"title": title, "link": download_url, "size": size, "seeders": seeders, "leechers": leechers, "hash": ""}
                        if mode != "RSS":
                            logger.debug("Found result: {0} with {1} seeders and {2} leechers".format(title, seeders, leechers))

                        items.append(item)
                    except (AttributeError, TypeError, KeyError, ValueError):
                        continue

            # For each search mode sort all the items by seeders if available
            items.sort(key=lambda d: try_int(d.get("seeders", 0)), reverse=True)
            results += items

        return results

    @staticmethod
    def _check_auth_from_data(data):
        if not all([isinstance(data, dict), data.pop("status", 200) != 401, data.pop("message", "") != "Invalid API key"]):

            logger.warning("Invalid api key. Check your settings")
            return False

        return True



import os

from . import generic


class PS3Metadata(generic.GenericMetadata):
    """
    Metadata generation class for Sony PS3.

    The following file structure is used:

    show_root/cover.jpg                         (poster)
    show_root/Season ##/filename.ext            (*)
    show_root/Season ##/filename.ext.cover.jpg  (episode thumb)
    """

    def __init__(
        self,
        show_metadata=False,
        episode_metadata=False,
        fanart=False,
        poster=False,
        banner=False,
        episode_thumbnails=False,
        season_posters=False,
        season_banners=False,
        season_all_poster=False,
        season_all_banner=False,
    ):

        super().__init__(
            show_metadata, episode_metadata, fanart, poster, banner, episode_thumbnails, season_posters, season_banners, season_all_poster, season_all_banner
        )

        self.name = "Sony PS3"

        self.poster_name = "cover.jpg"

        # web-ui metadata template
        self.eg_show_metadata = "<i>not supported</i>"
        self.eg_episode_metadata = "<i>not supported</i>"
        self.eg_fanart = "<i>not supported</i>"
        self.eg_poster = "cover.jpg"
        self.eg_banner = "<i>not supported</i>"
        self.eg_episode_thumbnails = "Season##\\<i>filename</i>.ext.cover.jpg"
        self.eg_season_posters = "<i>not supported</i>"
        self.eg_season_banners = "<i>not supported</i>"
        self.eg_season_all_poster = "<i>not supported</i>"
        self.eg_season_all_banner = "<i>not supported</i>"

    # Override with empty methods for unsupported features
    def retrieveShowMetadata(self, folder):
        # no show metadata generated, we abort this lookup function
        return None, None, None

    def create_show_metadata(self, show_obj):
        pass

    def update_show_indexer_metadata(self, show_obj):
        pass

    def get_show_file_path(self, show_obj):
        pass

    def create_episode_metadata(self, ep_obj):
        pass

    def create_fanart(self, show_obj):
        pass

    def create_banner(self, show_obj):
        pass

    def create_season_posters(self, show_obj):
        pass

    def create_season_banners(self, ep_obj):
        pass

    def create_season_all_poster(self, show_obj):
        pass

    def create_season_all_banner(self, show_obj):
        pass

    @staticmethod
    def get_episode_thumb_path(ep_obj):
        """
        Returns the path where the episode thumbnail should be stored. Defaults to
        the same path as the episode file but with a .cover.jpg extension.

        ep_obj: a TVEpisode instance for which to create the thumbnail
        """
        if os.path.isfile(ep_obj.location):
            tbn_filename = ep_obj.location + ".cover.jpg"
        else:
            return None

        return tbn_filename


# present a standard "interface" from the module
metadata_class = PS3Metadata



"""
This module contains all the beekeeper classes that are used on the front end
to directly interface between the developer and the remote API.
"""

from __future__ import absolute_import, division
from __future__ import unicode_literals, print_function

import copy
from functools import partial
from keyword import iskeyword

from beekeeper.variables import Variables
from beekeeper.hive import Hive
from beekeeper.comms import Request

class Endpoint(object):

    """
    Contains the settings for an endpoint, as well as a backref to the API
    """

    def __init__(self, parent, path, **kwargs):
        self.parent = parent
        self.path = path
        self.vars = Variables(**kwargs.get('variables', {}))
        self.methods = kwargs.get('methods', ['GET'])
        self.mimetype = kwargs.get('mimetype', None)

    def variables(self):
        """
        Get API-level variables, and add in Endpoint-level variables.
        """
        return self.parent.variables().add(**self.vars)

    def url(self):
        """
        Combine the API-level root URL with the Endpoint's path.
        """
        return self.parent.base_url() + self.path

    def new_action(self, method='GET', **kwargs):
        """
        Create a new Action linked to this endpoint with the given args.
        """
        if method not in self.methods:
            raise TypeError('{} not in valid method(s): {}.'.format(method, self.methods))
        return Action(self, method, **kwargs)

    def format(self):
        """
        Get the Endpoint-level mimetype, deferring to the API level settings
        if the Endpoint object doesn't have a value.
        """
        if self.mimetype:
            return self.mimetype
        else:
            return self.parent.format()

class APIObject(object):

    """
    Holds Action objects in the appropriate namespace, and provides a __getitem__
    dundermethod so that we can subscript by object ID when such exists
    """

    def __init__(self, parent, actions, **kwargs):
        self._description = kwargs.get('description', None)
        self._id_variable = kwargs.get('id_variable', None)
        if iskeyword(self._id_variable):
            self._id_variable = '_' + self._id_variable
        self._actions = {}
        for name, action in actions.items():
            self.add_action(name, parent, action)

    def __getitem__(self, key):
        """
        Allows us to subscript, dictionary-style, on the object if we
        know what the object's unique key is.
        """
        if self._id_variable:
            return APIObjectInstance(self, key)
        raise TypeError('Object cannot be addressed by ID')

    def defined_actions(self):
        """
        Get a list of the available Actions on the APIObject.
        """
        return [name for name in self._actions]

    def add_action(self, name, parent, action):
        """
        Add a single Action to the APIObject.
        """
        if iskeyword(name):
            name = '_' + name
        self._actions[name] = parent.new_action(**action)
        setattr(self, name, self._actions[name].execute)

    def printed_out(self, name):
        """
        Create a string describing the APIObject and its children
        """
        out = ''
        out += '|\n'
        if self._id_variable:
            subs = '[{}]'.format(self._id_variable)
        else:
            subs = ''
        out += '|---{}{}\n'.format(name, subs)
        if self._description:
            out += '|   |   {}\n'.format(self._description)
        for name, action in self._actions.items():
            out += action.printed_out(name)
        return out

    def id_variable(self):
        """
        Get the name of the variable that this particular APIObject can be subscripted by
        """
        return self._id_variable

class APIObjectInstance(object):
    """
    Ephemeral class that gets created/destroyed when the developer subscripts
    an APIObject; basically, the point is to execute an action with the subscripted
    key passed to it, as below; the two statements are equivalent:

    >>> ExampleAPI.Objects[123].update(varname=value)

    >>> ExampleAPI.Objects.update(object_id=123, varname=value)

    If assigned to a variable, it can be used directly - generally useful when
    subscripting gives you access to an object that can be used to access other
    objects; an example from the MBTA API:

    >>> OrangeLine = mbta.Routes['Orange']

    >>> OrangeLine.stops()

    >>> OrangeLine.alerts()
    """

    def __init__(self, api_object, id_key):
        self._api_object = api_object
        self._id_key = id_key
        self._actions = api_object.defined_actions
        self._id_variable = api_object.id_variable()

    def __getattr__(self, name):
        """
        If the action is one available on the parent APIObject, generate a
        partial with the ID variable set to have the value passed during
        initialization.
        """
        action = getattr(self._api_object, name)
        return partial(action, **{self._id_variable: self._id_key})

class Action(object):

    """
    Holds action-level variables and provides the .execute() method
    """

    def __init__(self, endpoint, method, **kwargs):
        self.endpoint = endpoint
        self.method = method
        self.vars = Variables(**kwargs.get('variables', {}))
        self.mimetype = kwargs.get('mimetype', None)
        self.url = endpoint.url
        self.description = kwargs.get('description', None)
        self.traversal = kwargs.get('traverse', None)
        self.timeout = kwargs.get('timeout', 5)

    def variables(self):
        """
        Grab all the variables from the Endpoint and higher, and
        add the ones that exist on the Action level.
        """
        return self.endpoint.variables().add(**self.vars)

    def execute(self, *args, **kwargs):
        """
        Fill all variables from *args and **kwargs, build the request,
        and send it. If we set the _verbose kwarg to true, then we'll
        get a Response object back instead of loaded data.
        """
        _verbose = kwargs.pop('_verbose', False)
        return_full_object = kwargs.pop('return_full_object', False)
        variables = self.variables().fill(*args, **kwargs)
        return Request(self, variables).send(
            traversal=self.traversal,
            _verbose=_verbose,
            return_full_object=return_full_object,
            _timeout=self.timeout
        )

    def format(self):
        """
        Get the local directional MIME type; if it doesn't exist, defer
        to the Endpoint-level MIME type.
        """
        if self.mimetype:
            return self.mimetype
        else:
            return self.endpoint.format()

    def printed_out(self, name):
        """
        Create a string representation of the action
        """
        opt = self.variables().optional_namestring()
        req = self.variables().required_namestring()
        out = ''
        out += '|   |\n'
        out += '|   |---{}({}{})\n'.format(name, req, opt)
        if self.description:
            out += '|   |       {}\n'.format(self.description)
        return out

class API(object):

    """
    Holds global-level settings and provides initialization methods
    """

    def __init__(self, hive, *args, **kwargs):
        self._root = hive.get('root')
        self._mimetype = hive.get('mimetype', 'application/json')
        self._vars = Variables(
            variable_settings=hive.get('variable_settings', {}),
            **hive.get('variables', {})
            ).fill(*args, **kwargs)
        self._endpoints = {}
        self._description = hive.get('description', None)
        self._name = hive.get('name', None)
        self._objects = {}
        for name, endpoint in hive['endpoints'].items():
            self.add_endpoint(name, **endpoint)
        for name, obj in hive['objects'].items():
            self.add_object(name, obj)

    @classmethod
    def from_hive_file(cls, fname, *args, **kwargs):
        """
        Open a local JSON hive file and initialize from the hive contained
        in that file, paying attention to the version keyword argument.
        """
        version = kwargs.pop('version', None)
        require = kwargs.pop('require_https', True)
        return cls(Hive.from_file(fname, version, require), *args, **kwargs)

    @classmethod
    def from_remote_hive(cls, url, *args, **kwargs):
        """
        Download a JSON hive file from a URL, and initialize from it,
        paying attention to the version keyword argument.
        """
        version = kwargs.pop('version', None)
        require = kwargs.pop('require_https', False)
        return cls(Hive.from_url(url, version, require), *args, **kwargs)

    @classmethod
    def from_domain(cls, domain, *args, **kwargs):
        """
        Try to download the hive file from the domain using the defined
        beekeeper spec of domain/api/hive.json.
        """
        version = kwargs.pop('version', None)
        require = kwargs.pop('require_https', False)
        return cls(Hive.from_domain(domain, version, require), *args, **kwargs)

    def __repr__(self):
        out = ''
        req_var = self.variables().required_namestring()
        opt_var = self.variables().optional_namestring()
        out += '{}({}{})\n'.format(self._name, req_var, opt_var)
        if self._description:
            out += '|   {}\n'.format(self._description)
        for name, obj in self._objects.items():
            out += obj.printed_out(name)
        return out

    def variables(self):
        """
        Return a copy of the API-level variables.
        """
        return copy.deepcopy(self._vars)

    def add_endpoint(self, name, **kwargs):
        """
        Add an endpoint with the given name to the API.
        """
        self._endpoints[name] = Endpoint(self, **kwargs)

    def add_object(self, name, obj):
        """
        Initialize an APIObject with the given name and make it available
        using dot notation from the top-level namespace.
        """
        if iskeyword(name):
            name = '_' + name
        setattr(self, name, APIObject(self, **obj))
        self._objects[name] = getattr(self, name)

    def new_action(self, endpoint, **kwargs):
        """
        Initialize a new Action linked to the named Endpoint that's
        a member of the API.
        """
        return self._endpoints[endpoint].new_action(**kwargs)

    def format(self):
        """
        Provide the API-level MIME type.
        """
        return self._mimetype

    def base_url(self):
        """
        Provides the API base URL.
        """
        return self._root



import datetime
import threading
import time

import sickchill
from sickchill import logger, settings
from sickchill.helper.exceptions import CantRefreshShowException, CantUpdateShowException
from sickchill.oldbeard import db, network_timezones, ui


class ShowUpdater(object):
    def __init__(self):
        self.lock = threading.Lock()
        self.amActive = False

        self.seven_days = 7 * 24 * 60 * 60

    def run(self, force=False):
        if self.amActive:
            return

        self.amActive = True
        try:
            logger.info("ShowUpdater for tvdb Api V3 starting")

            cache_db_con = db.DBConnection("cache.db")
            for index, provider in sickchill.indexer:
                database_result = cache_db_con.select("SELECT `time` FROM lastUpdate WHERE provider = ?", [provider.name])
                last_update = int(database_result[0][0]) if database_result else 0
                network_timezones.update_network_dict()
                update_timestamp = int(time.time())
                updated_shows = []

                if last_update:
                    logger.info("Last update: {}".format(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(last_update))))

                    current_check = update_timestamp
                    while current_check >= last_update:

                        try:
                            TvdbData = sickchill.indexer[1].updates(fromTime=current_check - self.seven_days, toTime=current_check)
                            TvdbData.series()
                            updated_shows.extend([d["id"] for d in TvdbData.series])
                        except Exception as error:
                            logger.info(str(error))

                        current_check -= self.seven_days - 1
                else:
                    logger.info(_("No last update time from the cache, so we do a full update for all shows"))

                pi_list = []
                for cur_show in settings.showList:
                    try:
                        cur_show.nextEpisode()

                        skip_update = False
                        # Skip ended shows until interval is met
                        if cur_show.status == "Ended" and settings.ENDED_SHOWS_UPDATE_INTERVAL != 0:  # 0 is always
                            if settings.ENDED_SHOWS_UPDATE_INTERVAL == -1:  # Never
                                skip_update = True
                            if (
                                datetime.datetime.today() - datetime.datetime.fromordinal(cur_show.last_update_indexer or 1)
                            ).days < settings.ENDED_SHOWS_UPDATE_INTERVAL:
                                skip_update = True

                        # Just update all of the shows for now until they fix the updates api
                        # When last_update is not set from the cache or the show was in the tvdb updated list we update the show
                        if not last_update or (cur_show.indexerid in updated_shows and not skip_update):
                            pi_list.append(settings.showQueueScheduler.action.update_show(cur_show, True))
                        else:
                            pi_list.append(settings.showQueueScheduler.action.refresh_show(cur_show, force))
                    except (CantUpdateShowException, CantRefreshShowException) as error:
                        logger.info(_("Automatic update failed: {0}").format(str(error)))

                ui.ProgressIndicators.setIndicator("dailyUpdate", ui.QueueProgressIndicator("Daily Update", pi_list))

                if database_result:
                    cache_db_con.action("UPDATE lastUpdate SET `time` = ? WHERE provider = ?", [str(update_timestamp), provider.name])
                else:
                    cache_db_con.action("INSERT INTO lastUpdate (time, provider) VALUES (?, ?)", [str(update_timestamp), provider.name])
        except Exception as error:
            logger.exception(str(error))

        self.amActive = False

    @staticmethod
    def request_hook(response, **kwargs):
        logger.info("{0} URL: {1} [Status: {2}]".format(response.request.method, response.request.url, response.status_code))

    def __del__(self):
        pass



import unittest

import sickchill.oldbeard.providers
from sickchill import settings
from sickchill.oldbeard import common as common
from sickchill.providers.GenericProvider import GenericProvider
from sickchill.tv import TVEpisode, TVShow
from tests import conftest

TESTS = {
    "Game of Thrones": {
        "tvdbid": 121361,
        "s": 5,
        "e": [10],
        "s_strings": [{"Season": ["Game of Thrones S05"]}],
        "e_strings": [{"Episode": ["Game of Thrones S05E10"]}],
    }
}


class SearchTest(conftest.SickChillTestDBCase):
    """
    Test search
    """

    def __init__(self, something):
        super().__init__(something)


def generator(cur_data, cur_name, cur_provider):
    """
    Generate test

    :param cur_data:
    :param cur_name:
    :param cur_provider:
    :return:
    """

    def do_test(self):
        """
        Test to perform
        """
        show = TVShow(1, int(cur_data["tvdbid"]))
        show.name = cur_name
        show.quality = common.ANY | common.Quality.UNKNOWN | common.Quality.RAWHDTV
        show.saveToDB()
        settings.showList.append(show)

        for ep_number in cur_data["e"]:
            episode = TVEpisode(show, cur_data["s"], ep_number)
            episode.status = common.WANTED

            # We aren't updating scene numbers, so fake it here
            episode.scene_season = cur_data["s"]
            episode.scene_episode = ep_number

            episode.saveToDB()

            cur_provider.show = show
            season_strings = cur_provider.get_season_search_strings(episode)
            episode_strings = cur_provider.get_episode_search_strings(episode)

            fail = False
            cur_string = ""
            for cur_string in season_strings, episode_strings:
                if not all([isinstance(cur_string, list), isinstance(cur_string[0], dict)]):
                    print("{0} is using a wrong string format!".format(cur_provider.name))
                    print(cur_string)
                    fail = True
                    continue

            if fail:
                continue

            try:
                assert season_strings == cur_data["s_strings"]
                assert episode_strings == cur_data["e_strings"]
            except AssertionError:
                print("{0} is using a wrong string format!".format(cur_provider.name))
                print(cur_string)
                continue

            search_strings = episode_strings[0]
            # search_strings.update(season_strings[0])
            # search_strings.update({"RSS":['']})

            # print(search_strings)

            if not cur_provider.public:
                continue

            items = cur_provider.search(search_strings)
            if not items:
                print("No results from cur_provider?")
                continue

            title, url = cur_provider._get_title_and_url(items[0])
            for word in show.name.split(" "):
                if not word.lower() in title.lower():
                    print("Show cur_name not in title: {0}. URL: {1}".format(title, url))
                    continue

            if not url:
                print("url is empty")
                continue

            quality = cur_provider.get_quality(items[0])
            size = cur_provider._get_size(items[0])

            if not show.quality & quality:
                print("Quality not in common.ANY, {0!r} {1}".format(quality, size))
                continue

    return do_test


if __name__ == "__main__":
    print("==================")
    print("STARTING - Search TESTS")
    print("==================")
    print("######################################################################")
    # create the test methods
    for forceSearch in (True, False):
        for name, data in TESTS.items():
            filename = name.replace(" ", "_")

            for provider in sickchill.oldbeard.providers.sortedProviderList():
                if provider.provider_type == GenericProvider.TORRENT:
                    if forceSearch:
                        test_name = "test_manual_{0}_{1}_{2}".format(filename, data["tvdbid"], provider.name)
                    else:
                        test_name = "test_{0}_{1}_{2}".format(filename, data["tvdbid"], provider.name)
                    test = generator(data, name, provider)
                    setattr(SearchTest, test_name, test)

    SUITE = unittest.TestLoader().loadTestsFromTestCase(SearchTest)
    unittest.TextTestRunner(verbosity=2).run(SUITE)



# -*- coding: utf-8 -*-

"""
This module implements the Updates functionality of TheTVDb API.
Allows to retrieve updated series in a timespan.

See [Updates API section](https://api.thetvdb.com/swagger#!/Updates)
"""

from .base import TVDB

class Updates(TVDB):
    """
    Updates class to retrieve updated series in a timespan.
    Requires `fromTime` that is the starting date of the updates.
    """
    _BASE_PATH = 'updated'
    _URLS = {
        'query': '/query',
        'update_params': '/query/params'
    }

    def __init__(self, fromTime, toTime='', language=''):
        """
        Initialize the Updates class.
        `fromTime` is the epoch time to start your date range to search for.
        You can set `toTime` to set the epoch time to end your date range. 
        Must be one week from `fromTime`.
        You can also set `language` with a language id to get the result 
        in the specific language.
        """
        self._FILTERS = {}
        self.update_filters(fromTime, toTime, language)

    def update_filters(self, fromTime='', toTime='', language=''):
        """
        Updates the filters for the updates class.

        `fromTime` is the epoch time to start your date range to search for.
        You can set `toTime` to set the epoch time to end your date range. 
        Must be one week from `fromTime`.
        You can also set `language` with a language id to get the result 
        in the specific language.
        """
        if fromTime:
            self._FILTERS['fromTime'] = fromTime
        if fromTime:
            self._FILTERS['toTime'] = toTime
        if language:
            self._set_language(language)

    def series(self):
        """
        Get the series updated in the set time span (maximom one week from fromTime).

        Returns a list of series updated in the timespan with basic info
        """
        path = self._get_path('query')
        
        response = self._GET(path, params=self._FILTERS)
        self._set_attrs_to_values({'series': response})
        return response

    def update_params(self):
        """
        Get the filters available for the updated api.

        Returns a list of filters available for updates.
        """
        path = self._get_path('update_params')
        
        response = self._GET(path)
        self._set_attrs_to_values({'update_params': response})
        return response



import datetime
import threading
import time
import traceback

from .. import logger


class Scheduler(threading.Thread):
    def __init__(
        self,
        action,
        cycleTime=datetime.timedelta(minutes=10),
        run_delay=datetime.timedelta(minutes=0),
        start_time=None,
        threadName="ScheduledThread",
        silent=True,
    ):
        super().__init__()

        self.run_delay = run_delay
        if start_time is None:
            self.lastRun = datetime.datetime.now() + self.run_delay - cycleTime
        else:
            # Set last run to the last full hour
            temp_now = datetime.datetime.now()
            self.lastRun = datetime.datetime(temp_now.year, temp_now.month, temp_now.day, temp_now.hour, 0, 0, 0) + self.run_delay - cycleTime
        self.action = action
        self.cycleTime = cycleTime
        self.start_time = start_time

        self.name = threadName
        self.silent = silent
        self.stop = threading.Event()
        self.force = False
        self.enable = False

    def timeLeft(self):
        """
        Check how long we have until we run again
        :return: timedelta
        """
        if self.is_alive():
            if self.start_time is None:
                delta = datetime.datetime.now() - self.lastRun
                return (self.cycleTime - delta, self.cycleTime)[delta > self.cycleTime]
            else:
                time_now = datetime.datetime.now()
                start_time_today = datetime.datetime.combine(time_now.date(), self.start_time)
                start_time_tomorrow = start_time_today + datetime.timedelta(days=1)
                if time_now.hour >= self.start_time.hour:
                    return start_time_tomorrow - time_now
                elif time_now.hour < self.start_time.hour:
                    return start_time_today - time_now
        else:
            return datetime.timedelta(seconds=0)

    def forceRun(self):
        if not self.action.amActive:
            self.force = True
            return True
        return False

    def run(self):
        """
        Runs the thread
        """
        try:
            while not self.stop.is_set():
                if self.enable:
                    current_time = datetime.datetime.now()
                    should_run = False
                    # Is self.force enable
                    if self.force:
                        should_run = True
                    # check if interval has passed
                    elif current_time - self.lastRun >= self.cycleTime:
                        # check if wanting to start around certain time taking interval into account
                        if self.start_time is not None:
                            hour_diff = current_time.time().hour - self.start_time.hour
                            if not hour_diff < 0 and hour_diff < self.cycleTime.seconds / 3600:
                                should_run = True
                            else:
                                # set lastRun to only check start_time after another cycleTime
                                self.lastRun = current_time
                        else:
                            should_run = True

                    if should_run:
                        self.lastRun = current_time
                        if not self.silent:
                            logger.debug("Starting new thread: " + self.name)
                        self.action.run(self.force)

                    if self.force:
                        self.force = False

                time.sleep(1)
            # exiting thread
            self.stop.clear()
        except Exception as e:
            logger.exception("Exception generated in thread " + self.name + ": " + str(e))
            logger.debug(repr(traceback.format_exc()))



import re
from urllib.parse import urljoin

import validators

from sickchill import logger
from sickchill.helper.common import convert_size, try_int
from sickchill.oldbeard import tvcache
from sickchill.oldbeard.bs4_parser import BS4Parser
from sickchill.providers.torrent.TorrentProvider import TorrentProvider


class Provider(TorrentProvider):
    def __init__(self):

        super().__init__("Torrent9")

        self.public = True
        self.minseed = 0
        self.minleech = 0
        self._original_url = "https://www.torrent9.one/"
        self._custom_url = None
        self._used_url = None
        self._recheck_url = True

        self.proper_strings = ["PROPER", "REPACK"]
        self.cache = tvcache.TVCache(self)

    def _retrieve_dllink_from_url(self, inner_url, _type="torrent"):
        data = self.get_url(inner_url, returns="text")
        res = {
            "torrent": "",
            "magnet": "",
        }
        with BS4Parser(data, "html5lib") as html:
            download_btns = html.findAll("div", {"class": "download-btn"})
            for btn in download_btns:
                link = btn.find("a")["href"]
                if link.startswith("magnet"):
                    res["magnet"] = link
                else:
                    res["torrent"] = link
        return res[_type]

    def _get_custom_url(self):
        return self._custom_url

    def _set_custom_url(self, url):
        if self._custom_url != url:
            self._custom_url = url
            self._recheck_url = True

    def _get_provider_url(self):
        if self._recheck_url:
            if self.custom_url:
                if validators.url(self.custom_url):
                    self._used_url = self.custom_url
                else:
                    logger.warning("Invalid custom url set, please check your settings")

            self._used_url = self._original_url

        return self._used_url

    def _set_provider_url(self, url):
        self._used_url = url

    url = property(_get_provider_url, _set_provider_url)
    custom_url = property(_get_custom_url, _set_custom_url)

    def search(self, search_strings, age=0, ep_obj=None):
        results = []
        for mode in search_strings:
            items = []
            logger.debug(_("Search Mode: {mode}".format(mode=mode)))
            for search_string in {*search_strings[mode]}:
                if mode == "Season":
                    search_string = re.sub(r"(.*)S0?", r"\1Saison ", search_string)

                if mode != "RSS":
                    logger.debug(_("Search String: {search_string}".format(search_string=search_string)))

                    search_url = self.url
                    post_data = {"torrentSearch": search_string}
                else:
                    search_url = self.url + "/torrents_series.html"
                    post_data = None

                data = self.get_url(search_url, post_data, returns="text")
                if not data:
                    continue

                with BS4Parser(data, "html5lib") as html:
                    torrent_table = html.find("div", {"class": "table-responsive"})
                    if torrent_table:
                        torrent_rows = torrent_table.findAll("tr")
                    else:
                        torrent_rows = None

                    if not torrent_rows:
                        continue
                    for result in torrent_rows:
                        try:
                            title = result.find("a").get_text(strip=False).replace("HDTV", "HDTV x264-Torrent9")
                            title = re.sub(r" Saison", " Season", title, flags=re.I)
                            tmp = result.find("a")["href"]
                            download_url = urljoin(self.url, self._retrieve_dllink_from_url(urljoin(self.url, tmp)))
                            if not all([title, download_url]):
                                continue

                            seeders = try_int(result.find(class_="seed_ok").get_text(strip=True))
                            leechers = try_int(result.find_all("td")[3].get_text(strip=True))
                            if seeders < self.minseed or leechers < self.minleech:
                                if mode != "RSS":
                                    logger.debug(
                                        "Discarding torrent because it doesn't meet the minimum seeders or leechers: {0} (S:{1} L:{2})".format(
                                            title, seeders, leechers
                                        )
                                    )
                                continue

                            torrent_size = result.find_all("td")[1].get_text(strip=True)

                            units = ["o", "Ko", "Mo", "Go", "To", "Po"]
                            size = convert_size(torrent_size, units=units) or -1

                            item = {"title": title, "link": download_url, "size": size, "seeders": seeders, "leechers": leechers, "hash": ""}
                            if mode != "RSS":
                                logger.debug("Found result: {0} with {1} seeders and {2} leechers".format(title, seeders, leechers))

                            items.append(item)
                        except Exception:
                            continue

            # For each search mode sort all the items by seeders if available
            items.sort(key=lambda d: try_int(d.get("seeders", 0)), reverse=True)
            results += items

        return results



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2019-11-05

@author: immoroz
'''

base = [
    ["щойно", "через мить"],
    ["%s секунду тому", "через %s секунду", "%s секунди тому", "через %s секунди", "%s секунд тому", "через %s секунд"],
    ["хвилину тому", "через хвилину"],
    ["%s хвилину тому", "через %s хвилину", "%s хвилини тому", "через %s хвилини", "%s хвилин тому", "через %s минут"],
    ["годину тому", "через годину"],
    ["%s годину тому", "через %s годину", "%s години тому", "через %s години", "%s годин тому", "через %s годин"],
    ["вчора", "завтра"],
    ["%s день тому", "через %s день", "%s дні тому", "через %s дні", "%s днів тому", "через %s днів"],
    ["тиждень тому", "через тиждень"],
    ["%s тиждень тому", "через %s тиждень", "%s тиждні тому", "через %s тиждні", "%s тижднів тому", "через %s тижднів"],
    ["місяць тому", "через місяць"],
    ["%s місяць тому", "через %s місяць", "%s місяці тому", "через %s місяці", "%s місяців тому", "через %s місяців"],
    ["рік тому", "через рік"],
    ["%s рік тому", "через %s рік", "%s роки тому", "через %s роки", "%s років тому", "через %s років"],
]


def generate(row, y):
    def formatting(time):
        if row % 2 == 0:
            return base[row][y]
        if time > 20: # for 21,32,43 but not for 11, 12, 13
            time = time % 10
        if time == 1:
            return base[row][y]
        elif time in (2, 3, 4):
            return base[row][y + 2]
        else:
            return base[row][y + 4]
    return formatting


LOCALE = generate



#!/usr/bin/python

from xml.dom.minidom import parseString

try:
        from http.client import HTTPSConnection
except ImportError:
        from httplib import HTTPSConnection

try:
        from urllib.parse import urlencode
except ImportError:
        from urllib import urlencode

__version__ = "1.0"

API_SERVER = 'www.notifymyandroid.com' 
ADD_PATH   = '/publicapi/notify'

USER_AGENT="PyNMA/v%s"%__version__

def uniq_preserve(seq): # Dave Kirby
    # Order preserving
    seen = set()
    return [x for x in seq if x not in seen and not seen.add(x)]

def uniq(seq):
    # Not order preserving
    return list({}.fromkeys(seq).keys())

class PyNMA(object):
    """PyNMA(apikey=[], developerkey=None)
takes 2 optional arguments:
 - (opt) apykey:      might me a string containing 1 key or an array of keys
 - (opt) developerkey: where you can store your developer key
"""

    def __init__(self, apikey=[], developerkey=None):
        self._developerkey = None
        self.developerkey(developerkey)
        if apikey:
            if type(apikey) == str:
                apikey = [apikey]
        self._apikey          = uniq(apikey)

    def addkey(self, key):
        "Add a key (register ?)"
        if type(key) == str:
            if not key in self._apikey:
                self._apikey.append(key)
        elif type(key) == list:
            for k in key:
                if not k in self._apikey:
                    self._apikey.append(k)

    def delkey(self, key):
        "Removes a key (unregister ?)"
        if type(key) == str:
            if key in self._apikey:
                self._apikey.remove(key)
        elif type(key) == list:
            for k in key:
                if key in self._apikey:
                    self._apikey.remove(k)

    def developerkey(self, developerkey):
        "Sets the developer key (and check it has the good length)"
        if type(developerkey) == str and len(developerkey) == 48:
            self._developerkey = developerkey

    def push(self, application="", event="", description="", url="", contenttype=None, priority=0, batch_mode=False, html=False):
        """Pushes a message on the registered API keys.
takes 5 arguments:
 - (req) application: application name [256]
 - (req) event:       event name       [1000]
 - (req) description: description      [10000]
 - (opt) url:         url              [512]
 - (opt) contenttype: Content Type (act: None (plain text) or text/html)
 - (opt) priority:    from -2 (lowest) to 2 (highest) (def:0)
 - (opt) batch_mode:  push to all keys at once (def:False)
 - (opt) html:        shortcut for contenttype=text/html
Warning: using batch_mode will return error only if all API keys are bad
 cf: http://nma.usk.bz/api.php
"""
        datas = {
            'application': application[:256].encode('utf8'),
            'event':       event[:1024].encode('utf8'),
            'description': description[:10000].encode('utf8'),
            'priority':    priority
        }

        if url:
            datas['url'] = url[:512]
        
        if contenttype == "text/html" or html == True: # Currently only accepted content type
            datas['content-type'] = "text/html"
        
        if self._developerkey:
            datas['developerkey'] = self._developerkey

        results = {}

        if not batch_mode:
            for key in self._apikey:
                datas['apikey'] = key
                res = self.callapi('POST', ADD_PATH, datas)
                results[key] = res
        else:
            datas['apikey'] = ",".join(self._apikey)
            res = self.callapi('POST', ADD_PATH, datas)
            results[datas['apikey']] = res
        return results
        
    def callapi(self, method, path, args):
        headers = { 'User-Agent': USER_AGENT }
        if method == "POST":
            headers['Content-type'] = "application/x-www-form-urlencoded"
        http_handler = HTTPSConnection(API_SERVER)
        http_handler.request(method, path, urlencode(args), headers)
        resp = http_handler.getresponse()

        try:
            res = self._parse_reponse(resp.read())
        except Exception as e:
            res = {'type':    "pynmaerror",
                   'code':    600,
                   'message': str(e)
                   }
            pass
        
        return res

    def _parse_reponse(self, response):
        root = parseString(response).firstChild
        for elem in root.childNodes:
            if elem.nodeType == elem.TEXT_NODE: continue
            if elem.tagName == 'success':
                res = dict(list(elem.attributes.items()))
                res['message'] = ""
                res['type']    = elem.tagName
                return res
            if elem.tagName == 'error':
                res = dict(list(elem.attributes.items()))
                res['message'] = elem.firstChild.nodeValue
                res['type']    = elem.tagName
                return res
                                        
    



import json
import ntpath
import os
import posixpath

from sickchill import logger, settings
from sickchill.helper import episode_num, try_int
from sickchill.helper.exceptions import CantRefreshShowException, CantUpdateShowException
from sickchill.oldbeard import db, subtitles as subtitle_module, ui
from sickchill.oldbeard.common import Overview, Quality, SNATCHED
from sickchill.show.Show import Show
from sickchill.views.common import PageTemplate
from sickchill.views.home import Home, WebRoot
from sickchill.views.routes import Route


@Route("/manage(/?.*)", name="manage:main")
class Manage(Home, WebRoot):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def index(self, *args, **kwargs):
        t = PageTemplate(rh=self, filename="manage.mako")
        return t.render(title=_("Mass Update"), header=_("Mass Update"), topmenu="manage", controller="manage", action="index")

    @staticmethod
    def showEpisodeStatuses(indexer_id, whichStatus):
        status_list = [int(whichStatus)]
        if status_list[0] == SNATCHED:
            status_list = Quality.SNATCHED + Quality.SNATCHED_PROPER + Quality.SNATCHED_BEST

        main_db_con = db.DBConnection()
        cur_show_results = main_db_con.select(
            "SELECT season, episode, name FROM tv_episodes WHERE showid = ? AND season != 0 AND status IN ({0})".format(",".join(["?"] * len(status_list))),
            [int(indexer_id)] + status_list,
        )

        result = {}
        for cur_result in cur_show_results:
            cur_season = int(cur_result["season"])
            cur_episode = int(cur_result["episode"])

            if cur_season not in result:
                result[cur_season] = {}

            result[cur_season][cur_episode] = cur_result["name"]

        return json.dumps(result)

    def episodeStatuses(self, whichStatus=None):
        if whichStatus:
            status_list = [int(whichStatus)]
            if status_list[0] == SNATCHED:
                status_list = Quality.SNATCHED + Quality.SNATCHED_PROPER + Quality.SNATCHED_BEST
        else:
            status_list = []

        t = PageTemplate(rh=self, filename="manage_episodeStatuses.mako")

        # if we have no status then this is as far as we need to go
        if not status_list:
            return t.render(
                title=_("Episode Overview"),
                header=_("Episode Overview"),
                topmenu="manage",
                show_names=None,
                whichStatus=whichStatus,
                ep_counts=None,
                sorted_show_ids=None,
                controller="manage",
                action="episodeStatuses",
            )

        main_db_con = db.DBConnection()
        status_results = main_db_con.select(
            "SELECT show_name, tv_shows.indexer_id AS indexer_id FROM tv_episodes, tv_shows WHERE tv_episodes.status IN ({0}) AND season != 0 AND "
            "tv_episodes.showid = tv_shows.indexer_id ORDER BY show_name".format(",".join(["?"] * len(status_list))),
            status_list,
        )

        ep_counts = {}
        show_names = {}
        sorted_show_ids = []
        for cur_status_result in status_results:
            cur_indexer_id = int(cur_status_result["indexer_id"])
            if cur_indexer_id not in ep_counts:
                ep_counts[cur_indexer_id] = 1
            else:
                ep_counts[cur_indexer_id] += 1

            show_names[cur_indexer_id] = cur_status_result["show_name"]
            if cur_indexer_id not in sorted_show_ids:
                sorted_show_ids.append(cur_indexer_id)

        return t.render(
            title=_("Episode Overview"),
            header=_("Episode Overview"),
            topmenu="manage",
            whichStatus=whichStatus,
            show_names=show_names,
            ep_counts=ep_counts,
            sorted_show_ids=sorted_show_ids,
            controller="manage",
            action="episodeStatuses",
        )

    # noinspection PyUnusedLocal
    def changeEpisodeStatuses(self, oldStatus, newStatus, *args, **kwargs):
        status_list = [int(oldStatus)]
        if status_list[0] == SNATCHED:
            status_list = Quality.SNATCHED + Quality.SNATCHED_PROPER + Quality.SNATCHED_BEST

        to_change = {}

        # make a list of all shows and their associated args
        for arg in kwargs:
            indexer_id, what = arg.split("-")

            # we don't care about unchecked checkboxes
            if kwargs[arg] != "on":
                continue

            if indexer_id not in to_change:
                to_change[indexer_id] = []

            to_change[indexer_id].append(what)

        main_db_con = db.DBConnection()
        for cur_indexer_id in to_change:

            # get a list of all the eps we want to change if they just said "all"
            if "all" in to_change[cur_indexer_id]:
                all_eps_results = main_db_con.select(
                    "SELECT season, episode FROM tv_episodes WHERE status IN ({0}) AND season != 0 AND showid = ?".format(",".join(["?"] * len(status_list))),
                    status_list + [cur_indexer_id],
                )
                all_eps = [str(x["season"]) + "x" + str(x["episode"]) for x in all_eps_results]
                to_change[cur_indexer_id] = all_eps

            self.setStatus(cur_indexer_id, "|".join(to_change[cur_indexer_id]), newStatus, direct=True)

        return self.redirect("/manage/episodeStatuses/")

    @staticmethod
    def showSubtitleMissed(indexer_id, whichSubs):
        main_db_con = db.DBConnection()
        cur_show_results = main_db_con.select(
            "SELECT season, episode, name, subtitles FROM tv_episodes WHERE showid = ? {0} AND (status LIKE '%4' OR status LIKE '%6') and "
            "location != ''".format(("AND season != 0 ", "")[settings.SUBTITLES_INCLUDE_SPECIALS]),
            [int(indexer_id)],
        )
        result = {}
        for cur_result in cur_show_results:
            if whichSubs == "all":
                if not frozenset(subtitle_module.wanted_languages()).difference(cur_result["subtitles"].split(",")):
                    continue
            elif whichSubs in cur_result["subtitles"]:
                continue

            cur_season = int(cur_result["season"])
            cur_episode = int(cur_result["episode"])

            if cur_season not in result:
                result[cur_season] = {}

            if cur_episode not in result[cur_season]:
                result[cur_season][cur_episode] = {}

            result[cur_season][cur_episode]["name"] = cur_result["name"]

            result[cur_season][cur_episode]["subtitles"] = cur_result["subtitles"]

        return json.dumps(result)

    def subtitleMissed(self, whichSubs=None):
        t = PageTemplate(rh=self, filename="manage_subtitleMissed.mako")

        if not whichSubs:
            return t.render(
                whichSubs=whichSubs,
                title=_("Episode Overview"),
                header=_("Episode Overview"),
                topmenu="manage",
                show_names=None,
                ep_counts=None,
                sorted_show_ids=None,
                controller="manage",
                action="subtitleMissed",
            )

        main_db_con = db.DBConnection()
        status_results = main_db_con.select(
            "SELECT show_name, tv_shows.indexer_id as indexer_id, tv_episodes.subtitles subtitles "
            + "FROM tv_episodes, tv_shows "
            + "WHERE tv_shows.subtitles = 1 AND (tv_episodes.status LIKE '%4' OR tv_episodes.status LIKE '%6') AND tv_episodes.season != 0 "
            + "AND tv_episodes.location != '' AND tv_episodes.showid = tv_shows.indexer_id ORDER BY show_name"
        )

        ep_counts = {}
        show_names = {}
        sorted_show_ids = []
        for cur_status_result in status_results:
            if whichSubs == "all":
                if not frozenset(subtitle_module.wanted_languages()).difference(cur_status_result["subtitles"].split(",")):
                    continue
            elif whichSubs in cur_status_result["subtitles"]:
                continue

            cur_indexer_id = int(cur_status_result["indexer_id"])
            if cur_indexer_id not in ep_counts:
                ep_counts[cur_indexer_id] = 1
            else:
                ep_counts[cur_indexer_id] += 1

            show_names[cur_indexer_id] = cur_status_result["show_name"]
            if cur_indexer_id not in sorted_show_ids:
                sorted_show_ids.append(cur_indexer_id)

        return t.render(
            whichSubs=whichSubs,
            show_names=show_names,
            ep_counts=ep_counts,
            sorted_show_ids=sorted_show_ids,
            title=_("Missing Subtitles"),
            header=_("Missing Subtitles"),
            topmenu="manage",
            controller="manage",
            action="subtitleMissed",
        )

    # noinspection PyUnusedLocal
    def downloadSubtitleMissed(self, *args, **kwargs):
        to_download = {}

        # make a list of all shows and their associated args
        for arg in kwargs:
            indexer_id, what = arg.split("-")

            # we don't care about unchecked checkboxes
            if kwargs[arg] != "on":
                continue

            if indexer_id not in to_download:
                to_download[indexer_id] = []

            to_download[indexer_id].append(what)

        for cur_indexer_id in to_download:
            # get a list of all the eps we want to download subtitles if they just said "all"
            if "all" in to_download[cur_indexer_id]:
                main_db_con = db.DBConnection()
                all_eps_results = main_db_con.select(
                    "SELECT season, episode FROM tv_episodes WHERE (status LIKE '%4' OR status LIKE '%6') {0} AND showid = ? AND location != ''".format(
                        ("AND season != 0 ", "")[settings.SUBTITLES_INCLUDE_SPECIALS]
                    ),
                    [cur_indexer_id],
                )
                to_download[cur_indexer_id] = [str(x["season"]) + "x" + str(x["episode"]) for x in all_eps_results]

            for epResult in to_download[cur_indexer_id]:
                season, episode = epResult.split("x")

                show = Show.find(settings.showList, int(cur_indexer_id))
                show.getEpisode(season, episode).download_subtitles()

        return self.redirect("/manage/subtitleMissed/")

    def backlogShow(self, indexer_id):
        show_obj = Show.find(settings.showList, int(indexer_id))

        if show_obj:
            settings.backlogSearchScheduler.action.searchBacklog([show_obj])

        return self.redirect("/manage/backlogOverview/")

    def backlogOverview(self):
        t = PageTemplate(rh=self, filename="manage_backlogOverview.mako")

        showCounts = {}
        showCats = {}
        showSQLResults = {}

        main_db_con = db.DBConnection()
        for curShow in settings.showList:

            epCounts = {
                Overview.SKIPPED: 0,
                Overview.WANTED: 0,
                Overview.QUAL: 0,
                Overview.GOOD: 0,
                Overview.UNAIRED: 0,
                Overview.SNATCHED: 0,
                Overview.SNATCHED_PROPER: 0,
                Overview.SNATCHED_BEST: 0,
            }
            epCats = {}

            sql_results = main_db_con.select(
                "SELECT status, season, episode, name, airdate FROM tv_episodes WHERE tv_episodes.season IS NOT NULL "
                "AND tv_episodes.showid IN (SELECT tv_shows.indexer_id FROM tv_shows WHERE tv_shows.indexer_id = ? "
                "AND paused = 0) ORDER BY tv_episodes.season DESC, tv_episodes.episode DESC",
                [curShow.indexerid],
            )

            for curResult in sql_results:
                curEpCat = curShow.getOverview(curResult["status"], backlog=settings.BACKLOG_MISSING_ONLY)
                if curEpCat:

                    epCats["{ep}".format(ep=episode_num(curResult["season"], curResult["episode"]))] = curEpCat
                    epCounts[curEpCat] += 1

            showCounts[curShow.indexerid] = epCounts
            showCats[curShow.indexerid] = epCats
            showSQLResults[curShow.indexerid] = sql_results

        def showQualSnatched(show):
            return Quality.splitQuality(show.quality)[1]

        totalWanted = totalQual = totalQualSnatched = 0
        backLogShows = sorted(
            [
                x
                for x in settings.showList
                if (showCounts[x.indexerid][Overview.QUAL] or showCounts[x.indexerid][Overview.WANTED] or
                    (0, showCounts[x.indexerid][Overview.SNATCHED])[len(showQualSnatched(x)) > 0])
            ],
            key=lambda x: x.sort_name,
        )
        for curShow in backLogShows:
            totalWanted += showCounts[curShow.indexerid][Overview.WANTED]
            totalQual += showCounts[curShow.indexerid][Overview.QUAL]
            if showQualSnatched(curShow):
                totalQualSnatched += showCounts[curShow.indexerid][Overview.SNATCHED]

        return t.render(
            showCounts=showCounts,
            showCats=showCats,
            totalQual=totalQual,
            showQualSnatched=showQualSnatched,
            totalWanted=totalWanted,
            totalQualSnatched=totalQualSnatched,
            backLogShows=backLogShows,
            showSQLResults=showSQLResults,
            controller="manage",
            action="backlogOverview",
            title=_("Backlog Overview"),
            header=_("Backlog Overview"),
            topmenu="manage",
        )

    @staticmethod
    def __gooey_path(name, method):
        result = getattr(os.path, method)(name)
        if result == name or not result:
            result = getattr(ntpath, method)(name)
            if result == name or not result:
                result = getattr(posixpath, method)(name)

        return result

    # noinspection PyProtectedMember
    def massEdit(self, toEdit=None):
        t = PageTemplate(rh=self, filename="manage_massEdit.mako")

        if not toEdit:
            return self.redirect("/manage/")

        showIDs = toEdit.split("|")
        showList = []
        showNames = []
        for curID in showIDs:
            curID = int(curID)
            show_obj = Show.find(settings.showList, curID)
            if show_obj:
                showList.append(show_obj)
                showNames.append(show_obj.name)

        season_folders_all_same = True
        last_season_folders = None

        paused_all_same = True
        last_paused = None

        default_ep_status_all_same = True
        last_default_ep_status = None

        anime_all_same = True
        last_anime = None

        sports_all_same = True
        last_sports = None

        quality_all_same = True
        last_quality = None

        subtitles_all_same = True
        last_subtitles = None

        scene_all_same = True
        last_scene = None

        air_by_date_all_same = True
        last_air_by_date = None

        root_dir_list = []

        for curShow in showList:

            cur_root_dir = self.__gooey_path(curShow._location, "dirname")
            if cur_root_dir and cur_root_dir != curShow._location and cur_root_dir not in root_dir_list:
                root_dir_list.append(cur_root_dir)

            # if we know they're not all the same then no point even bothering
            if paused_all_same:
                # if we had a value already and this value is different then they're not all the same
                if last_paused not in (None, curShow.paused):
                    paused_all_same = False
                else:
                    last_paused = curShow.paused

            if default_ep_status_all_same:
                if last_default_ep_status not in (None, curShow.default_ep_status):
                    default_ep_status_all_same = False
                else:
                    last_default_ep_status = curShow.default_ep_status

            if anime_all_same:
                # if we had a value already and this value is different then they're not all the same
                if last_anime not in (None, curShow.is_anime):
                    anime_all_same = False
                else:
                    last_anime = curShow.anime

            if season_folders_all_same:
                if last_season_folders not in (None, curShow.season_folders):
                    season_folders_all_same = False
                else:
                    last_season_folders = curShow.season_folders

            if quality_all_same:
                if last_quality not in (None, curShow.quality):
                    quality_all_same = False
                else:
                    last_quality = curShow.quality

            if subtitles_all_same:
                if last_subtitles not in (None, curShow.subtitles):
                    subtitles_all_same = False
                else:
                    last_subtitles = curShow.subtitles

            if scene_all_same:
                if last_scene not in (None, curShow.scene):
                    scene_all_same = False
                else:
                    last_scene = curShow.scene

            if sports_all_same:
                if last_sports not in (None, curShow.sports):
                    sports_all_same = False
                else:
                    last_sports = curShow.sports

            if air_by_date_all_same:
                if last_air_by_date not in (None, curShow.air_by_date):
                    air_by_date_all_same = False
                else:
                    last_air_by_date = curShow.air_by_date

        default_ep_status_value = last_default_ep_status if default_ep_status_all_same else None
        paused_value = last_paused if paused_all_same else None
        anime_value = last_anime if anime_all_same else None
        season_folders_value = last_season_folders if season_folders_all_same else None
        quality_value = last_quality if quality_all_same else None
        subtitles_value = last_subtitles if subtitles_all_same else None
        scene_value = last_scene if scene_all_same else None
        sports_value = last_sports if sports_all_same else None
        air_by_date_value = last_air_by_date if air_by_date_all_same else None

        return t.render(
            showList=toEdit,
            showNames=showNames,
            default_ep_status_value=default_ep_status_value,
            paused_value=paused_value,
            anime_value=anime_value,
            season_folders_value=season_folders_value,
            quality_value=quality_value,
            subtitles_value=subtitles_value,
            scene_value=scene_value,
            sports_value=sports_value,
            air_by_date_value=air_by_date_value,
            root_dir_list=root_dir_list,
            title=_("Mass Edit"),
            header=_("Mass Edit"),
            controller="manage",
            action="massEdit",
            topmenu="manage",
        )

    # noinspection PyProtectedMember, PyUnusedLocal
    def massEditSubmit(
        self,
        paused=None,
        default_ep_status=None,
        anime=None,
        sports=None,
        scene=None,
        season_folders=None,
        quality_preset=None,
        subtitles=None,
        air_by_date=None,
        anyQualities=None,
        bestQualities=None,
        toEdit=None,
        *args,
        **kwargs,
    ):
        dir_map = {}
        for cur_arg in [x for x in kwargs if x.startswith("orig_root_dir_")]:
            dir_map[kwargs[cur_arg]] = kwargs[cur_arg.replace("orig_root_dir_", "new_root_dir_")]

        showIDs = toEdit.split("|")
        errors = []
        for curShow in showIDs:
            curErrors = []
            show_obj = Show.find(settings.showList, int(curShow or 0))
            if not show_obj:
                continue

            cur_root_dir = self.__gooey_path(show_obj._location, "dirname")
            cur_show_dir = self.__gooey_path(show_obj._location, "basename")
            if cur_root_dir and dir_map.get(cur_root_dir) and cur_root_dir != dir_map.get(cur_root_dir):
                new_show_dir = os.path.join(dir_map[cur_root_dir], cur_show_dir)
                logger.info("For show " + show_obj.name + " changing dir from " + show_obj._location + " to " + new_show_dir)
            else:
                new_show_dir = show_obj._location

            new_paused = ("off", "on")[(paused == "enable", show_obj.paused)[paused == "keep"]]
            new_default_ep_status = (default_ep_status, show_obj.default_ep_status)[default_ep_status == "keep"]
            new_anime = ("off", "on")[(anime == "enable", show_obj.anime)[anime == "keep"]]
            new_sports = ("off", "on")[(sports == "enable", show_obj.sports)[sports == "keep"]]
            new_scene = ("off", "on")[(scene == "enable", show_obj.scene)[scene == "keep"]]
            new_air_by_date = ("off", "on")[(air_by_date == "enable", show_obj.air_by_date)[air_by_date == "keep"]]
            new_season_folders = ("off", "on")[(season_folders == "enable", show_obj.season_folders)[season_folders == "keep"]]
            new_subtitles = ("off", "on")[(subtitles == "enable", show_obj.subtitles)[subtitles == "keep"]]

            if quality_preset == "keep":
                anyQualities, bestQualities = Quality.splitQuality(show_obj.quality)
            elif try_int(quality_preset, None):
                bestQualities = []

            exceptions_list = []

            curErrors += self.editShow(
                curShow,
                new_show_dir,
                anyQualities,
                bestQualities,
                exceptions_list,
                defaultEpStatus=new_default_ep_status,
                season_folders=new_season_folders,
                paused=new_paused,
                sports=new_sports,
                subtitles=new_subtitles,
                anime=new_anime,
                scene=new_scene,
                air_by_date=new_air_by_date,
                directCall=True,
            )

            if curErrors:
                logger.exception("Errors: " + str(curErrors))
                errors.append("<b>{0}:</b>\n<ul>".format(show_obj.name) + " ".join(["<li>{0}</li>".format(error) for error in curErrors]) + "</ul>")

        if len(errors) > 0:
            ui.notifications.error(
                _("{num_errors:d} error{plural} while saving changes:").format(num_errors=len(errors), plural="" if len(errors) == 1 else "s"), " ".join(errors)
            )

        return self.redirect("/manage/")

    def massUpdate(self, toUpdate=None, toRefresh=None, toRename=None, toDelete=None, toRemove=None, toMetadata=None, toSubtitle=None):

        toUpdate = toUpdate.split("|") if toUpdate else []
        toRefresh = toRefresh.split("|") if toRefresh else []
        toRename = toRename.split("|") if toRename else []
        toSubtitle = toSubtitle.split("|") if toSubtitle else []
        toDelete = toDelete.split("|") if toDelete else []
        toRemove = toRemove.split("|") if toRemove else []
        toMetadata = toMetadata.split("|") if toMetadata else []

        errors = []
        refreshes = []
        updates = []
        renames = []
        subtitles = []

        for curShowID in set(toUpdate + toRefresh + toRename + toSubtitle + toDelete + toRemove + toMetadata):

            if curShowID == "":
                continue

            show_obj = Show.find(settings.showList, int(curShowID))
            if not show_obj:
                continue

            if curShowID in toDelete:
                settings.showQueueScheduler.action.remove_show(show_obj, True)
                # don't do anything else if it's being deleted
                continue

            if curShowID in toRemove:
                settings.showQueueScheduler.action.remove_show(show_obj)
                # don't do anything else if it's being remove
                continue

            if curShowID in toUpdate:
                try:
                    settings.showQueueScheduler.action.update_show(show_obj, True)
                    updates.append(show_obj.name)
                except CantUpdateShowException as e:
                    errors.append(_("Unable to update show: {exception_format}").format(exception_format=e))

            # don't bother refreshing shows that were updated anyway
            if curShowID in toRefresh and curShowID not in toUpdate:
                try:
                    settings.showQueueScheduler.action.refresh_show(show_obj, force=True)
                    refreshes.append(show_obj.name)
                except CantRefreshShowException as e:
                    errors.append(_("Unable to refresh show {show_name}: {exception_format}").format(show_name=show_obj.name, exception_format=e))

            if curShowID in toRename:
                settings.showQueueScheduler.action.rename_show_episodes(show_obj)
                renames.append(show_obj.name)

            if curShowID in toSubtitle:
                settings.showQueueScheduler.action.download_subtitles(show_obj)
                subtitles.append(show_obj.name)

        if errors:
            ui.notifications.error(_("Errors encountered"), "<br >\n".join(errors))

        messageDetail = ""

        if updates:
            messageDetail += "<br><b>" + _("Updates") + "</b><br><ul><li>"
            messageDetail += "</li><li>".join(updates)
            messageDetail += "</li></ul>"

        if refreshes:
            messageDetail += "<br><b>" + _("Refreshes") + "</b><br><ul><li>"
            messageDetail += "</li><li>".join(refreshes)
            messageDetail += "</li></ul>"

        if renames:
            messageDetail += "<br><b>" + _("Renames") + "</b><br><ul><li>"
            messageDetail += "</li><li>".join(renames)
            messageDetail += "</li></ul>"

        if subtitles:
            messageDetail += "<br><b>" + _("Subtitles") + "</b><br><ul><li>"
            messageDetail += "</li><li>".join(subtitles)
            messageDetail += "</li></ul>"

        if updates + refreshes + renames + subtitles:
            ui.notifications.message(_("The following actions were queued") + ":", messageDetail)

        return self.redirect("/manage/")

    def failedDownloads(self, limit=100, toRemove=None):
        failed_db_con = db.DBConnection("failed.db")

        if limit == "0":
            sql_results = failed_db_con.select("SELECT * FROM failed")
        else:
            sql_results = failed_db_con.select("SELECT * FROM failed LIMIT ?", [limit])

        toRemove = toRemove.split("|") if toRemove else []

        for release in toRemove:
            failed_db_con.action("DELETE FROM failed WHERE failed.release = ?", [release])

        if toRemove:
            return self.redirect("/manage/failedDownloads/")

        t = PageTemplate(rh=self, filename="manage_failedDownloads.mako")

        return t.render(
            limit=limit,
            failedResults=sql_results,
            title=_("Failed Downloads"),
            header=_("Failed Downloads"),
            topmenu="manage",
            controller="manage",
            action="failedDownloads",
        )



import re
from xml.etree import ElementTree

from .. import logger
from . import classes, helpers
from .name_parser.parser import InvalidNameException, InvalidShowException, NameParser


def get_season_nzbs(name, url_data, season):
    """
    Split a season NZB into episodes

    :param name: NZB name
    :param url_data: URL to get data from
    :param season: Season to check
    :return: dict of (episode files, xml matches)
    """

    # TODO: clean up these regex'es, comment them, and make them all raw strings
    regex_string = {
        # Match the xmlns in an nzb
        # Example:  nzbElement.getchildren()[1].tag == '{http://www.newzbin.com/DTD/2003/nzb}file'
        #           regex match returns  'http://www.newzbin.com/DTD/2003/nzb'
        "nzb_xmlns": r"{(http://[\w_\./]+nzb)}file",
        "scene_name": r"([\w\._\ ]+)[\. ]S%02d[\. ]([\w\._\-\ ]+)[\- ]([\w_\-\ ]+?)",
        "episode": r"\.S%02d(?:[E0-9]+)\.[\w\._]+\-\w+",
    }

    try:
        show_xml = ElementTree.ElementTree(ElementTree.XML(url_data))
    except SyntaxError:
        logger.exception("Unable to parse the XML of " + name + ", not splitting it")
        return {}, ""

    nzb_element = show_xml.getroot()

    scene_name_match = re.search(regex_string["scene_name"] % season, name, re.I)
    if scene_name_match:
        show_name = scene_name_match.groups()[0]
    else:  # Make sure we aren't missing valid results after changing name_parser and the quality detection
        # Most of these will likely be invalid shows
        logger.debug("Unable to parse " + name + " into a scene name.")
        return {}, ""

    regex = "(" + re.escape(show_name) + regex_string["episode"] % season + ")"
    regex = regex.replace(" ", ".")

    ep_files = {}
    xmlns = None

    for cur_file in nzb_element.iter():
        xmlns_match = re.match(regex_string["nzb_xmlns"], cur_file.tag)
        if not xmlns_match:
            continue
        else:
            xmlns = xmlns_match.group(1)
        match = re.search(regex, cur_file.get("subject"), re.I)
        if not match:
            # regex couldn't match cur_file.get("subject")
            continue
        cur_ep = match.group(1)
        if cur_ep not in ep_files:
            ep_files[cur_ep] = [cur_file]
        else:
            ep_files[cur_ep].append(cur_file)
    # TODO: Decide what to do if we found multiple valid xmlns strings, should we only return the last???
    return ep_files, xmlns


def create_nzb_string(file_elements, xmlns):
    """
    Extract extra info from file_elements.

    :param file_elements: to be processed
    :param xmlns: the xml namespace to be used
    :return: string containing all extra info extracted from the file_elements
    """
    root_element = ElementTree.Element("nzb")
    if xmlns:
        root_element.set("xmlns", xmlns)

    for cur_file in file_elements:
        root_element.append(strip_xmlns(cur_file, xmlns))

    return ElementTree.tostring(root_element)


def save_nzb(nzb_name, nzb_string):
    """
    Save NZB to disk

    :param nzb_name: Filename/path to write to
    :param nzb_string: Content to write in file
    """
    try:
        with open(nzb_name + ".nzb", "w", encoding="utf-8") as nzb_fh:
            nzb_fh.write(nzb_string)

    except EnvironmentError as error:
        logger.exception("Unable to save NZB: " + str(error))


def strip_xmlns(element, xmlns):
    """
    Remove the xml namespace from the element's children.

    :param element: to be processed
    :param xmlns: xml namespace to be removed
    :return: processed element
    """
    element.tag = element.tag.replace("{" + xmlns + "}", "")
    for cur_child in element.getchildren():
        strip_xmlns(cur_child, xmlns)

    return element


def split_result(obj):
    """
    Split obj into separate episodes.

    :param obj: to search for results
    :return: a list of episode objects or an empty list
    """
    url_data = helpers.getURL(obj.url, session=helpers.make_session(), returns="content")
    if url_data is None:
        logger.warning("Unable to load url " + obj.url + ", can't download season NZB")
        return []

    # parse the season ep name
    try:
        parsed_obj = NameParser(False, showObj=obj.show).parse(obj.name)
    except (InvalidNameException, InvalidShowException) as error:
        logger.debug("{0}".format(error))
        return []

    # bust it up
    season = 1 if parsed_obj.season_number is None else parsed_obj.season_number

    separate_nzbs, xmlns = get_season_nzbs(obj.name, url_data, season)

    result_list = []

    # TODO: Re-evaluate this whole section
    #   If we have valid results and hit an exception, we ignore the results found so far.
    #   Maybe we should return the results found or possibly continue with the next iteration of the loop
    #   Also maybe turn this into a function and generate the results_list with a list comprehension instead
    for new_nzb in separate_nzbs:
        logger.debug("Split out " + new_nzb + " from " + obj.name)

        # parse the name
        try:
            parsed_obj = NameParser(False, showObj=obj.show).parse(new_nzb)
        except (InvalidNameException, InvalidShowException) as error:
            logger.debug("{0}".format(error))
            return []

        # make sure the result is sane
        if (parsed_obj.season_number != season) or (parsed_obj.season_number is None and season != 1):

            logger.warning("Found " + new_nzb + " inside " + obj.name + " but it doesn't seem to belong to the same season, ignoring it")
            continue
        elif not parsed_obj.episode_numbers:

            logger.warning("Found " + new_nzb + " inside " + obj.name + " but it doesn't seem to be a valid episode NZB, ignoring it")
            continue

        want_ep = True
        for ep_num in parsed_obj.episode_numbers:
            if not obj.extraInfo[0].wantEpisode(season, ep_num, obj.quality):
                logger.debug("Ignoring result: " + new_nzb)
                want_ep = False
                break
        if not want_ep:
            continue

        # get all the associated episode objects
        ep_obj_list = [obj.extraInfo[0].getEpisode(season, ep) for ep in parsed_obj.episode_numbers]

        # make a result
        cur_obj = classes.NZBDataSearchResult(ep_obj_list)
        cur_obj.name = new_nzb
        cur_obj.provider = obj.provider
        cur_obj.quality = obj.quality
        cur_obj.extraInfo = [create_nzb_string(separate_nzbs[new_nzb], xmlns)]

        result_list.append(cur_obj)

    return result_list



from twitter import Api, TwitterError
import requests


class Api(Api):
    def DebugEndpoint(self, verb=None, endpoint=None, data=None):
        """ Request a url and return raw data. For testing purposes only.

        Args:
            url:
                The web location we want to retrieve.
            verb:
                Either POST or GET.
            data:
                A dict of (str, unicode) key/value pairs.

        Returns:
            data
        """

        url = "{0}{1}".format(self.base_url, endpoint)

        if verb == 'POST':
            if 'media_ids' in data:
                url = self._BuildUrl(
                    url,
                    extra_params={
                        'media_ids': data['media_ids']
                    }
                )
                print('POSTing url:', url)
            if 'media' in data:
                try:
                    print('POSTing url:', url)
                    raw_data = requests.post(
                        url,
                        files=data,
                        auth=self.__auth,
                        timeout=self._timeout
                    )
                except requests.RequestException as e:
                    raise TwitterError(str(e))
            else:
                try:
                    print('POSTing url:', url)
                    raw_data = requests.post(
                        url,
                        data=data,
                        auth=self.__auth,
                        timeout=self._timeout
                    )
                except requests.RequestException as e:
                    raise TwitterError(str(e))
        if verb == 'GET':
            url = self._BuildUrl(url, extra_params=data)
            print('GETting url:', url)
            try:
                raw_data = requests.get(
                    url,
                    auth=self.__auth,
                    timeout=self._timeout)

            except requests.RequestException as e:
                raise TwitterError(str(e))
        return raw_data._content



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2017-8-30

@author: generated by @lolobosse script
'''
LOCALE = [
    ["just now", "right now"],
    ["%ss ago", "in %ss"],
    ["1m ago", "in 1m"],
    ["%sm ago", "in %sm"],
    ["1h ago", "in 1h"],
    ["%sh ago", "in %sh"],
    ["1d ago", "in 1d"],
    ["%sd ago", "in %sd"],
    ["1w ago", "in 1w"],
    ["%sw ago", "in %sw"],
    ["1mo ago", "in 1mo"],
    ["%smo ago", "in %smo"],
    ["1yr ago", "in 1yr"],
    ["%syr ago", "in %syr"]
]
        


import html
import os
import warnings

from sickchill import settings
from sickchill.oldbeard import common

# noinspection PyUnresolvedReferences
with warnings.catch_warnings():
    warnings.simplefilter("ignore")
    try:
        # noinspection PyUnresolvedReferences
        from pgi.repository import Notify
    except (ImportError, Exception):
        try:
            # noinspection PyUnresolvedReferences
            from gi.repository import Notify
        except (ImportError, Exception):
            Notify = None


# noinspection PyUnresolvedReferences
class Notifier(object):
    def __init__(self):
        self.notify_initialized = None
        if Notify:
            self.notify_initialized = Notify.init("SickChill")

    @staticmethod
    def diagnose():
        """
        Check the environment for reasons libnotify isn't working.  Return a
        user-readable message indicating possible issues.
        """
        if not Notify:
            return (
                "<p>Error: gir-notify isn't installed. On Ubuntu/Debian, install the "
                '<a href="apt:gir1.2-notify-0.7">gir1.2-notify-0.7</a> or '
                '<a href="apt:gir1.0-notify-0.4">gir1.0-notify-0.4</a> package.'
            )

        if "DISPLAY" not in os.environ and "DBUS_SESSION_BUS_ADDRESS" not in os.environ:
            return (
                "<p>Error: Environment variables DISPLAY and DBUS_SESSION_BUS_ADDRESS "
                "aren't set.  libnotify will only work when you run SickChill "
                "from a desktop login."
            )

        try:
            import dbus
        except (ImportError, Exception):
            dbus = None

        if dbus:
            try:
                bus = dbus.SessionBus()
            except dbus.DBusException as e:
                return ("<p>Error: unable to connect to D-Bus session bus: <code>{}</code>." "<p>Are you running SickChill in a desktop session?").format(
                    html.escape(e)
                )
            try:
                bus.get_object("org.freedesktop.Notifications", "/org/freedesktop/Notifications")
            except dbus.DBusException as e:
                return (
                    "<p>Error: there doesn't seem to be a notification daemon available: <code>{}</code> "
                    "<p>Try installing notification-daemon or notify-osd."
                ).format(html.escape(e))

        return "<p>Error: Unable to send notification."

    def notify_snatch(self, ep_name):
        if settings.LIBNOTIFY_NOTIFY_ONSNATCH:
            self._notify(common.notifyStrings[common.NOTIFY_SNATCH], ep_name)

    def notify_download(self, ep_name):
        if settings.LIBNOTIFY_NOTIFY_ONDOWNLOAD:
            self._notify(common.notifyStrings[common.NOTIFY_DOWNLOAD], ep_name)

    def notify_subtitle_download(self, ep_name, lang):
        if settings.LIBNOTIFY_NOTIFY_ONSUBTITLEDOWNLOAD:
            self._notify(common.notifyStrings[common.NOTIFY_SUBTITLE_DOWNLOAD], ep_name + ": " + lang)

    def notify_git_update(self, new_version="??"):
        if settings.USE_LIBNOTIFY:
            update_text = common.notifyStrings[common.NOTIFY_GIT_UPDATE_TEXT]
            title = common.notifyStrings[common.NOTIFY_GIT_UPDATE]
            self._notify(title, update_text + new_version)

    def notify_login(self, ipaddress=""):
        if settings.USE_LIBNOTIFY:
            update_text = common.notifyStrings[common.NOTIFY_LOGIN_TEXT]
            title = common.notifyStrings[common.NOTIFY_LOGIN]
            self._notify(title, update_text.format(ipaddress))

    def test_notify(self):
        return self._notify("Test notification", "This is a test notification from SickChill", force=True)

    def _notify(self, title, message, force=False):
        if self.notify_initialized and settings.USE_LIBNOTIFY | force:
            icon = os.path.join(settings.PROG_DIR, "gui", settings.GUI_NAME, "images", "ico", "favicon-120.png")
            # noinspection PyBroadException
            try:
                n = Notify.Notification.new(title, message, icon)
                return n.show()
            except Exception:
                return False
